# Copyright 2023 The KerasNLP Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for EditDistance."""

import tensorflow as tf
from tensorflow import keras

from keras_nlp.metrics import EditDistance


class EditDistanceTest(tf.test.TestCase):
    def test_initialization(self):
        edit_distance = EditDistance()
        result = edit_distance.result()

        self.assertEqual(result, 0.0)

    def test_1d_list_input_normalize(self):
        edit_distance = EditDistance()
        y_true = "the tiny little cat was found under the big funny bed".split()
        y_pred = "the cat was found under the bed".split()

        edit_distance_val = edit_distance(y_true, y_pred)
        self.assertAlmostEqual(edit_distance_val.numpy(), 0.364, delta=1e-3)

    def test_2d_list_input_normalize(self):
        edit_distance = EditDistance()
        y_true = [
            "the tiny little cat was found under the big funny bed".split(),
            "it is sunny today".split(),
        ]
        y_pred = [
            "the cat was found under the bed".split(),
            "it is sunny but with a hint of cloud cover".split(),
        ]

        edit_distance_val = edit_distance(y_true, y_pred)
        self.assertAlmostEqual(edit_distance_val.numpy(), 0.733, delta=1e-3)

    def test_rank_1_tensor_input_normalize(self):
        edit_distance = EditDistance()
        y_true = tf.strings.split(
            "the tiny little cat was found under the big funny bed"
        )
        y_pred = tf.strings.split("the cat was found under the bed")

        edit_distance_val = edit_distance(y_true, y_pred)
        self.assertAlmostEqual(edit_distance_val.numpy(), 0.364, delta=1e-3)

    def test_rank_2_tensor_input_normalize(self):
        edit_distance = EditDistance()
        y_true = tf.strings.split(
            [
                "the tiny little cat was found under the big funny bed",
                "it is sunny today",
            ]
        )
        y_pred = tf.strings.split(
            [
                "the cat was found under the bed",
                "it is sunny but with a hint of cloud cover",
            ]
        )

        edit_distance_val = edit_distance(y_true, y_pred)
        self.assertAlmostEqual(edit_distance_val.numpy(), 0.733, delta=1e-3)

    def test_rank_1_tensor_input_normalize_false(self):
        edit_distance = EditDistance(normalize=False)
        y_true = tf.strings.split(
            "the tiny little cat was found under the big funny bed"
        )
        y_pred = tf.strings.split("the cat was found under the bed")

        edit_distance_val = edit_distance(y_true, y_pred)
        self.assertAlmostEqual(edit_distance_val.numpy(), 4.0, delta=1e-3)

    def test_rank_2_tensor_input_normalize_false(self):
        edit_distance = EditDistance(normalize=False)
        y_true = tf.strings.split(
            [
                "the tiny little cat was found under the big funny bed",
                "it is sunny today",
            ]
        )
        y_pred = tf.strings.split(
            [
                "the cat was found under the bed",
                "it is sunny but with a hint of cloud cover",
            ]
        )

        edit_distance_val = edit_distance(y_true, y_pred)
        self.assertAlmostEqual(edit_distance_val.numpy(), 5.5, delta=1e-3)

    def test_model_compile_normalize(self):
        inputs = keras.Input(shape=(None,), dtype="string")
        outputs = tf.strings.lower(inputs)
        model = keras.Model(inputs, outputs)

        model.compile(metrics=[EditDistance()])

        x = tf.strings.split(
            ["the tiny little cat was found under the big funny bed"]
        )
        y = tf.strings.split(["the cat was found under the bed"])

        output = model.evaluate(y, x, return_dict=True)

        self.assertAlmostEqual(output["edit_distance"], 0.364, delta=1e-3)

    def test_model_compile_normalize_false(self):
        inputs = keras.Input(shape=(None,), dtype="string")
        outputs = tf.strings.lower(inputs)
        model = keras.Model(inputs, outputs)

        model.compile(metrics=[EditDistance(normalize=False)])

        x = tf.strings.split(
            ["the tiny little cat was found under the big funny bed"]
        )
        y = tf.strings.split(["the cat was found under the bed"])

        output = model.evaluate(y, x, return_dict=True)

        self.assertAlmostEqual(output["edit_distance"], 4.0, delta=1e-3)

    def test_reset_state_normalize(self):
        edit_distance = EditDistance()
        y_true = tf.strings.split(
            [
                "the tiny little cat was found under the big funny bed",
                "it is sunny today",
            ]
        )
        y_pred = tf.strings.split(
            [
                "the cat was found under the bed",
                "it is sunny but with a hint of cloud cover",
            ]
        )

        edit_distance.update_state(y_true, y_pred)
        edit_distance_val = edit_distance.result()
        self.assertNotEqual(edit_distance_val.numpy(), 0.0)

        edit_distance.reset_state()
        edit_distance_val = edit_distance.result()
        self.assertEqual(edit_distance_val, 0.0)

    def test_update_state_normalize(self):
        edit_distance = EditDistance()
        y_true_1 = tf.strings.split(
            [
                "the tiny little cat was found under the big funny bed",
                "it is sunny today",
            ]
        )
        y_pred_1 = tf.strings.split(
            [
                "the cat was found under the bed",
                "it is sunny but with a hint of cloud cover",
            ]
        )

        edit_distance.update_state(y_true_1, y_pred_1)
        edit_distance_val = edit_distance.result()
        self.assertAlmostEqual(edit_distance_val.numpy(), 0.733, delta=1e-3)

        y_true_2 = tf.strings.split(["what is your favourite show"])
        y_pred_2 = tf.strings.split(["my favourite show is silicon valley"])

        edit_distance.update_state(y_true_2, y_pred_2)
        edit_distance_val = edit_distance.result()
        self.assertAlmostEqual(edit_distance_val.numpy(), 0.85, delta=1e-3)

    def test_update_state_normalize_false(self):
        edit_distance = EditDistance(normalize=False)
        y_true_1 = tf.strings.split(
            [
                "the tiny little cat was found under the big funny bed",
                "it is sunny today",
            ]
        )
        y_pred_1 = tf.strings.split(
            [
                "the cat was found under the bed",
                "it is sunny but with a hint of cloud cover",
            ]
        )

        edit_distance.update_state(y_true_1, y_pred_1)
        edit_distance_val = edit_distance.result()
        self.assertAlmostEqual(edit_distance_val.numpy(), 5.5, delta=1e-3)

        y_true_2 = tf.strings.split(["what is your favourite show"])
        y_pred_2 = tf.strings.split(["my favourite show is silicon valley"])

        edit_distance.update_state(y_true_2, y_pred_2)
        edit_distance_val = edit_distance.result()
        self.assertAlmostEqual(edit_distance_val.numpy(), 5.667, delta=1e-3)

    def test_merge_state_normalize(self):
        edit_distance_1 = EditDistance()
        edit_distance_2 = EditDistance()

        y_true_1 = tf.strings.split(
            [
                "the tiny little cat was found under the big funny bed",
                "it is sunny today",
            ]
        )
        y_pred_1 = tf.strings.split(
            [
                "the cat was found under the bed",
                "it is sunny but with a hint of cloud cover",
            ]
        )

        y_true_2 = tf.strings.split(["what is your favourite show"])
        y_pred_2 = tf.strings.split(["my favourite show is silicon valley"])

        y_true_3 = tf.strings.split(["lorem ipsum dolor sit amet"])
        y_pred_3 = tf.strings.split(["lorem ipsum is simply dummy text"])

        edit_distance_1.update_state(y_true_1, y_pred_1)
        edit_distance_1.update_state(y_true_2, y_pred_2)
        edit_distance_val = edit_distance_1.result()
        self.assertAlmostEqual(edit_distance_val.numpy(), 0.85, delta=1e-3)

        edit_distance_2.update_state(y_true_3, y_pred_3)
        edit_distance_val = edit_distance_2.result()
        self.assertAlmostEqual(edit_distance_val.numpy(), 0.8, delta=1e-3)

        merged_edit_distance = EditDistance()
        merged_edit_distance.merge_state([edit_distance_1, edit_distance_2])
        edit_distance_val = merged_edit_distance.result()
        self.assertAlmostEqual(edit_distance_val.numpy(), 0.84, delta=1e-3)

    def test_merge_state_normalize_false(self):
        edit_distance_1 = EditDistance(normalize=False)
        edit_distance_2 = EditDistance(normalize=False)

        y_true_1 = tf.strings.split(
            [
                "the tiny little cat was found under the big funny bed",
                "it is sunny today",
            ]
        )
        y_pred_1 = tf.strings.split(
            [
                "the cat was found under the bed",
                "it is sunny but with a hint of cloud cover",
            ]
        )

        y_true_2 = tf.strings.split(["what is your favourite show"])
        y_pred_2 = tf.strings.split(["my favourite show is silicon valley"])

        y_true_3 = tf.strings.split(["lorem ipsum dolor sit amet"])
        y_pred_3 = tf.strings.split(["lorem ipsum is simply dummy text"])

        edit_distance_1.update_state(y_true_1, y_pred_1)
        edit_distance_1.update_state(y_true_2, y_pred_2)
        edit_distance_val = edit_distance_1.result()
        self.assertAlmostEqual(edit_distance_val.numpy(), 5.667, delta=1e-3)

        edit_distance_2.update_state(y_true_3, y_pred_3)
        edit_distance_val = edit_distance_2.result()
        self.assertAlmostEqual(edit_distance_val.numpy(), 4.0, delta=1e-3)

        merged_edit_distance = EditDistance(normalize=False)
        merged_edit_distance.merge_state([edit_distance_1, edit_distance_2])
        edit_distance_val = merged_edit_distance.result()
        self.assertAlmostEqual(edit_distance_val.numpy(), 5.25, delta=1e-3)

    def test_get_config(self):
        rouge = EditDistance(
            normalize=False,
            dtype=tf.float32,
            name="edit_distance_test",
        )

        config = rouge.get_config()
        expected_config_subset = {
            "normalize": False,
        }
        self.assertEqual(config, {**config, **expected_config_subset})



# Copyright 2023 The KerasNLP Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for RandomDeletion Layer."""

import tensorflow as tf
from tensorflow import keras

from keras_nlp.layers import RandomDeletion


class RandomDeletionTest(tf.test.TestCase):
    def test_shape_and_output_from_word_deletion(self):
        keras.utils.set_random_seed(1337)
        inputs = ["Hey I like", "Keras and Tensorflow"]
        split = tf.strings.split(inputs)
        augmenter = RandomDeletion(rate=0.4, max_deletions=1, seed=42)
        augmented = augmenter(split)
        output = tf.strings.reduce_join(augmented, separator=" ", axis=-1)
        self.assertAllEqual(output.shape, tf.convert_to_tensor(inputs).shape)
        exp_output = [b"I like", b"and Tensorflow"]
        self.assertAllEqual(output, exp_output)

    def test_shape_and_output_from_character_swaps(self):
        keras.utils.set_random_seed(1337)
        inputs = ["Hey I like", "Keras and Tensorflow"]
        split = tf.strings.unicode_split(inputs, "UTF-8")
        augmenter = RandomDeletion(rate=0.4, max_deletions=1, seed=42)
        augmented = augmenter(split)
        output = tf.strings.reduce_join(augmented, axis=-1)
        self.assertAllEqual(output.shape, tf.convert_to_tensor(inputs).shape)
        exp_output = [b"Hey I lie", b"Keras and Tensoflow"]
        self.assertAllEqual(output, exp_output)

    def test_with_integer_tokens(self):
        keras.utils.set_random_seed(1337)
        inputs = tf.constant([[1, 2], [3, 4]])
        augmenter = RandomDeletion(rate=0.4, max_deletions=4, seed=42)
        output = augmenter(inputs)
        self.assertAllEqual(output.to_tensor().shape[0], inputs.shape[0])
        exp_output = [[2], [4]]
        self.assertAllEqual(output, exp_output)

    def test_skip_options(self):
        keras.utils.set_random_seed(1337)
        augmenter = RandomDeletion(
            rate=0.4, max_deletions=1, seed=42, skip_list=["Tensorflow", "like"]
        )
        inputs = ["Hey I like", "Keras and Tensorflow"]
        split = tf.strings.split(inputs)
        augmented = augmenter(split)
        output = tf.strings.reduce_join(augmented, separator=" ", axis=-1)
        self.assertAllEqual(output.shape, tf.convert_to_tensor(inputs).shape)
        exp_output = [b"I like", b"and Tensorflow"]
        self.assertAllEqual(output, exp_output)

        def skip_fn(word):
            if word == "Tensorflow" or word == "like":
                return True
            return False

        augmenter = RandomDeletion(
            rate=0.4, max_deletions=1, seed=42, skip_fn=skip_fn
        )
        augmented = augmenter(split)
        output = tf.strings.reduce_join(augmented, separator=" ", axis=-1)
        self.assertAllEqual(output.shape, tf.convert_to_tensor(inputs).shape)
        exp_output = [b"Hey like", b"Keras Tensorflow"]
        self.assertAllEqual(output, exp_output)

        def skip_py_fn(word):
            if word == "Tensorflow" or word == "like":
                return True
            return False

        augmenter = RandomDeletion(
            rate=0.4, max_deletions=1, seed=42, skip_py_fn=skip_py_fn
        )
        augmented = augmenter(split)
        output = tf.strings.reduce_join(augmented, separator=" ", axis=-1)
        self.assertAllEqual(output.shape, tf.convert_to_tensor(inputs).shape)
        exp_output = [b"Hey like", b"Keras Tensorflow"]

    def test_get_config_and_from_config(self):
        augmenter = RandomDeletion(rate=0.4, max_deletions=1, seed=42)

        expected_config_subset = {"max_deletions": 1, "rate": 0.4, "seed": 42}

        config = augmenter.get_config()

        self.assertEqual(config, {**config, **expected_config_subset})

        restored_augmenter = RandomDeletion.from_config(
            config,
        )

        self.assertEqual(
            restored_augmenter.get_config(),
            {**config, **expected_config_subset},
        )

    def test_augment_first_batch_second(self):
        keras.utils.set_random_seed(1337)
        augmenter = RandomDeletion(rate=0.4, max_deletions=1, seed=42)
        inputs = ["Hey I like", "Keras and Tensorflow"]
        split = tf.strings.split(inputs)
        ds = tf.data.Dataset.from_tensor_slices(split)
        ds = ds.map(augmenter)
        ds = ds.apply(tf.data.experimental.dense_to_ragged_batch(2))
        output = ds.take(1).get_single_element()

        exp_output = [[b"I", b"like"], [b"Keras", b"and", b"Tensorflow"]]
        self.assertAllEqual(output, exp_output)

        def skip_fn(word):
            return tf.strings.regex_full_match(word, r"\pP")

        def skip_py_fn(word):
            return len(word) < 4

        augmenter = RandomDeletion(
            rate=0.8, max_deletions=1, seed=42, skip_fn=skip_fn
        )
        ds = tf.data.Dataset.from_tensor_slices(split)
        ds = ds.map(augmenter)
        ds = ds.apply(tf.data.experimental.dense_to_ragged_batch(2))
        output = ds.take(1).get_single_element()
        exp_output = [[b"I", b"like"], [b"and", b"Tensorflow"]]
        self.assertAllEqual(output, exp_output)

        augmenter = RandomDeletion(
            rate=0.8, max_deletions=1, seed=42, skip_py_fn=skip_py_fn
        )
        ds = tf.data.Dataset.from_tensor_slices(split)
        ds = ds.map(augmenter)
        ds = ds.apply(tf.data.experimental.dense_to_ragged_batch(2))
        output = ds.take(1).get_single_element()
        exp_output = [[b"Hey", b"I", b"like"], [b"and", b"Tensorflow"]]
        self.assertAllEqual(output, exp_output)

    def test_batch_first_augment_second(self):
        keras.utils.set_random_seed(1337)
        augmenter = RandomDeletion(rate=0.4, max_deletions=1, seed=42)
        inputs = ["Hey I like", "Keras and Tensorflow"]
        split = tf.strings.split(inputs)
        ds = tf.data.Dataset.from_tensor_slices(split)
        ds = ds.batch(5).map(augmenter)
        output = ds.take(1).get_single_element()

        exp_output = [[b"I", b"like"], [b"and", b"Tensorflow"]]
        self.assertAllEqual(output, exp_output)

        def skip_fn(word):
            return tf.strings.regex_full_match(word, r"\pP")

        def skip_py_fn(word):
            return len(word) < 4

        augmenter = RandomDeletion(
            rate=0.8, max_deletions=1, seed=42, skip_fn=skip_fn
        )
        ds = tf.data.Dataset.from_tensor_slices(split)
        ds = ds.batch(5).map(augmenter)
        output = ds.take(1).get_single_element()
        exp_output = [[b"I", b"like"], [b"and", b"Tensorflow"]]
        self.assertAllEqual(output, exp_output)

        augmenter = RandomDeletion(
            rate=0.8, max_deletions=1, seed=42, skip_py_fn=skip_py_fn
        )
        ds = tf.data.Dataset.from_tensor_slices(split)
        ds = ds.batch(5).map(augmenter)
        output = ds.take(1).get_single_element()
        exp_output = [[b"Hey", b"I", b"like"], [b"and", b"Tensorflow"]]
        self.assertAllEqual(output, exp_output)

    def test_functional_model(self):
        keras.utils.set_random_seed(1337)
        input_data = tf.constant(["Hey I like", "Keras and Tensorflow"])
        augmenter = RandomDeletion(rate=0.4, max_deletions=1, seed=42)
        inputs = tf.keras.Input(dtype="string", shape=())
        outputs = augmenter(tf.strings.split(inputs))
        model = tf.keras.Model(inputs, outputs)
        model_output = model(input_data)
        self.assertAllEqual(
            model_output, [[b"I", b"like"], [b"and", b"Tensorflow"]]
        )



# Copyright 2022 The KerasNLP Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for RoBERTa masked language model."""

import os

import tensorflow as tf
from absl.testing import parameterized
from tensorflow import keras

from keras_nlp.models.roberta.roberta_backbone import RobertaBackbone
from keras_nlp.models.roberta.roberta_masked_lm import RobertaMaskedLM
from keras_nlp.models.roberta.roberta_masked_lm_preprocessor import (
    RobertaMaskedLMPreprocessor,
)
from keras_nlp.models.roberta.roberta_tokenizer import RobertaTokenizer


class RobertaMaskedLMTest(tf.test.TestCase, parameterized.TestCase):
    def setUp(self):
        self.backbone = RobertaBackbone(
            vocabulary_size=1000,
            num_layers=2,
            num_heads=2,
            hidden_dim=64,
            intermediate_dim=128,
            max_sequence_length=128,
        )
        self.vocab = {
            "<s>": 0,
            "<pad>": 1,
            "</s>": 2,
            "Ġair": 3,
            "plane": 4,
            "Ġat": 5,
            "port": 6,
            "Ġkoh": 7,
            "li": 8,
            "Ġis": 9,
            "Ġthe": 10,
            "Ġbest": 11,
            "<mask>": 12,
        }

        merges = ["Ġ a", "Ġ t", "Ġ k", "Ġ i", "Ġ b", "Ġa i", "p l", "n e"]
        merges += ["Ġa t", "p o", "r t", "o h", "l i", "Ġi s", "Ġb e", "s t"]
        merges += ["Ġt h", "Ġai r", "pl a", "Ġk oh", "Ġth e", "Ġbe st", "po rt"]
        merges += ["pla ne"]
        self.merges = merges
        self.preprocessor = RobertaMaskedLMPreprocessor(
            RobertaTokenizer(vocabulary=self.vocab, merges=self.merges),
            sequence_length=8,
            mask_selection_length=2,
        )
        self.masked_lm = RobertaMaskedLM(
            self.backbone,
            preprocessor=self.preprocessor,
        )
        self.masked_lm_no_preprocessing = RobertaMaskedLM(
            self.backbone,
            preprocessor=None,
        )

        self.raw_batch = tf.constant(
            [
                " airplane at airport",
                " the airplane is the best",
                " the best airport",
                " kohli is the best",
            ]
        )
        self.preprocessed_batch = self.preprocessor(self.raw_batch)[0]
        self.raw_dataset = tf.data.Dataset.from_tensor_slices(
            self.raw_batch
        ).batch(2)
        self.preprocessed_dataset = self.raw_dataset.map(self.preprocessor)

    def test_valid_call_masked_lm(self):
        self.masked_lm(self.preprocessed_batch)

    @parameterized.named_parameters(
        ("jit_compile_false", False), ("jit_compile_true", True)
    )
    def test_roberta_masked_lm_predict(self, jit_compile):
        self.masked_lm.compile(jit_compile=jit_compile)
        self.masked_lm.predict(self.raw_batch)

    @parameterized.named_parameters(
        ("jit_compile_false", False), ("jit_compile_true", True)
    )
    def test_roberta_masked_lm_predict_no_preprocessing(self, jit_compile):
        self.masked_lm_no_preprocessing.compile(jit_compile=jit_compile)
        self.masked_lm_no_preprocessing.predict(self.preprocessed_batch)

    @parameterized.named_parameters(
        ("jit_compile_false", False), ("jit_compile_true", True)
    )
    def test_roberta_masked_lm_fit(self, jit_compile):
        self.masked_lm.compile(
            loss=keras.losses.SparseCategoricalCrossentropy(from_logits=True),
            jit_compile=jit_compile,
        )
        self.masked_lm.fit(self.raw_dataset)

    @parameterized.named_parameters(
        ("jit_compile_false", False), ("jit_compile_true", True)
    )
    def test_roberta_masked_lm_fit_no_preprocessing(self, jit_compile):
        self.masked_lm_no_preprocessing.compile(
            loss=keras.losses.SparseCategoricalCrossentropy(from_logits=True),
            jit_compile=jit_compile,
        )
        self.masked_lm_no_preprocessing.fit(self.preprocessed_dataset)

    @parameterized.named_parameters(
        ("tf_format", "tf", "model"),
        ("keras_format", "keras_v3", "model.keras"),
    )
    def test_saved_model(self, save_format, filename):
        save_path = os.path.join(self.get_temp_dir(), filename)
        self.masked_lm.save(save_path, save_format=save_format)
        restored_model = keras.models.load_model(save_path)

        # Check we got the real object back.
        self.assertIsInstance(restored_model, RobertaMaskedLM)

        model_output = self.masked_lm(self.preprocessed_batch)
        restored_output = restored_model(self.preprocessed_batch)

        self.assertAllClose(model_output, restored_output)



# Copyright 2023 The KerasNLP Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import tensorflow as tf

import keras_nlp.layers.transformer_layer_utils as utils


class TransformerEncoderTest(tf.test.TestCase):
    def test_compute_causal_mask(self):
        inputs = tf.random.uniform(shape=[1, 2, 2])
        mask = utils.compute_causal_mask(inputs)
        self.assertTrue((mask.numpy() == [[1, 0], [1, 1]]).all())

    def test_merge_padding_and_attention_mask(self):
        padding_mask = tf.convert_to_tensor([[1, 1, 0]])
        attention_mask = tf.convert_to_tensor([[0, 0, 1], [0, 1, 0], [1, 0, 0]])
        inputs = tf.random.uniform(shape=[1, 3, 2])
        merged_mask = utils.merge_padding_and_attention_mask(
            inputs,
            padding_mask,
            attention_mask,
        )
        self.assertTrue(
            (merged_mask.numpy() == [[0, 0, 0], [0, 1, 0], [1, 0, 0]]).all()
        )



# Copyright 2023 The KerasNLP Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""DeBERTa tokenizer."""

import copy

from tensorflow import keras

from keras_nlp.models.deberta_v3.deberta_v3_presets import backbone_presets
from keras_nlp.tokenizers.sentence_piece_tokenizer import SentencePieceTokenizer
from keras_nlp.utils.python_utils import classproperty


@keras.utils.register_keras_serializable(package="keras_nlp")
class DebertaV3Tokenizer(SentencePieceTokenizer):
    """DeBERTa tokenizer layer based on SentencePiece.

    This tokenizer class will tokenize raw strings into integer sequences and
    is based on `keras_nlp.tokenizers.SentencePieceTokenizer`. Unlike the
    underlying tokenizer, it will check for all special tokens needed by
    DeBERTa models and provides a `from_preset()` method to automatically
    download a matching vocabulary for a DeBERTa preset.

    This tokenizer does not provide truncation or padding of inputs. It can be
    combined with a `keras_nlp.models.DebertaV3Preprocessor` layer for input
    packing.

    If input is a batch of strings (rank > 0), the layer will output a
    `tf.RaggedTensor` where the last dimension of the output is ragged.

    If input is a scalar string (rank == 0), the layer will output a dense
    `tf.Tensor` with static shape `[None]`.

    Args:
        proto: Either a `string` path to a SentencePiece proto file, or a
            `bytes` object with a serialized SentencePiece proto. See the
            [SentencePiece repository](https://github.com/google/sentencepiece)
            for more details on the format.

    Examples:

    ```python
    tokenizer = keras_nlp.models.DebertaV3Tokenizer(proto="model.spm")

    # Batched inputs.
    tokenizer(["the quick brown fox", "the earth is round"])

    # Unbatched inputs.
    tokenizer("the quick brown fox")

    # Detokenization.
    tokenizer.detokenize(tf.constant([[1, 4, 9, 5, 7, 2]]))
    ```
    """

    def __init__(self, proto, **kwargs):
        super().__init__(proto=proto, **kwargs)

        # Check for necessary special tokens.
        cls_token = "[CLS]"
        sep_token = "[SEP]"
        pad_token = "[PAD]"
        for token in [cls_token, pad_token, sep_token]:
            if token not in self.get_vocabulary():
                raise ValueError(
                    f"Cannot find token `'{token}'` in the provided "
                    f"`vocabulary`. Please provide `'{token}'` in your "
                    "`vocabulary` or use a pretrained `vocabulary` name."
                )

        self.cls_token_id = self.token_to_id(cls_token)
        self.sep_token_id = self.token_to_id(sep_token)
        self.pad_token_id = self.token_to_id(pad_token)

    @classproperty
    def presets(cls):
        return copy.deepcopy(backbone_presets)



# Copyright 2023 The KerasNLP Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""GPT-2 preprocessing layers."""

import copy

from tensorflow import keras

from keras_nlp.models.gpt2.gpt2_presets import backbone_presets
from keras_nlp.tokenizers.byte_pair_tokenizer import BytePairTokenizer
from keras_nlp.utils.python_utils import classproperty


@keras.utils.register_keras_serializable(package="keras_nlp")
class GPT2Tokenizer(BytePairTokenizer):
    """A GPT-2 tokenizer using Byte-Pair Encoding subword segmentation.

    This tokenizer class will tokenize raw strings into integer sequences and
    is based on `keras_nlp.tokenizers.BytePairTokenizer`. Unlike the
    underlying tokenizer, it will check for all special tokens needed by GPT-2
    models and provides a `from_preset()` method to automatically download
    a matching vocabulary for a GPT-2 preset.

    This tokenizer does not provide truncation or padding of inputs.

    If input is a batch of strings (rank > 0), the layer will output a
    `tf.RaggedTensor` where the last dimension of the output is ragged.

    If input is a scalar string (rank == 0), the layer will output a dense
    `tf.Tensor` with static shape `[None]`.

    Args:
        vocabulary: string or dict, maps token to integer ids. If it is a
            string, it should be the file path to a json file.
        merges: string or list, contains the merge rule. If it is a string,
            it should be the file path to merge rules. The merge rule file
            should have one merge rule per line. Every merge rule contains
            merge entities separated by a space.

    Examples:

    Batched inputs.
    >>> vocab = {"<|endoftext|>": 0, "reful":1, "gent": 2, "Ġafter": 3}
    >>> vocab = {**vocab, **{"noon": 4, "Ġsun": 5}}
    >>> merges = ["Ġ a", "Ġ s", "r e", "f u", "g e", "n t", "e r", "n o", "o n"]
    >>> merges += ["i g", "h t", "Ġs u", "Ġa f", "ge nt", "no on", "re fu"]
    >>> merges += ["Ġsu n", "Ġaf t", "refu l", "Ġaft er"] # Ġ for whitespace
    >>> inputs = [" afternoon sun", "refulgent sun"]
    >>> tokenizer = keras_nlp.models.GPT2Tokenizer(
    ...     vocabulary=vocab,
    ...     merges=merges,
    ... )
    >>> tokenizer(inputs)
    <tf.RaggedTensor [[3, 4, 5], [1, 2, 5]]>

    Unbatched input.
    >>> vocab = {"<|endoftext|>": 0, "Ġafter": 1, "noon": 2, "Ġsun": 3}
    >>> merges = ["Ġ a", "Ġ s", "e r", "n o", "o n", "i g", "h t", "Ġs u"]
    >>> merges += ["Ġa f", "no on", "Ġsu n", "Ġaf t", "Ġaft er"]
    >>> inputs = " afternoon sun"
    >>> tokenizer = keras_nlp.models.GPT2Tokenizer(
    ...     vocabulary=vocab,
    ...     merges=merges,
    ... )
    >>> tokenizer(inputs)
    <tf.Tensor: shape=(3,), dtype=int32, numpy=array([1, 2, 3], dtype=int32)>

    Detokenization.
    >>> vocab = {"<|endoftext|>": 0, "Ġafter": 1, "noon": 2, "Ġsun": 3}
    >>> merges = ["Ġ a", "Ġ s", "e r", "n o", "o n", "i g", "h t", "Ġs u"]
    >>> merges += ["Ġa f", "no on", "Ġsu n", "Ġaf t", "Ġaft er"]
    >>> inputs = " afternoon sun"
    >>> tokenizer = keras_nlp.models.GPT2Tokenizer(
    ...     vocabulary=vocab,
    ...     merges=merges,
    ... )
    >>> tokenizer.detokenize(tokenizer.tokenize(inputs)).numpy().decode('utf-8')
    ' afternoon sun'
    """

    def __init__(
        self,
        vocabulary,
        merges,
        **kwargs,
    ):
        super().__init__(
            vocabulary=vocabulary,
            merges=merges,
            **kwargs,
        )

        # Check for necessary special tokens.
        end_token = "<|endoftext|>"

        if end_token not in self.get_vocabulary():
            raise ValueError(
                f"Cannot find token `'{end_token}'` in the provided "
                f"`vocabulary`. Please provide `'{end_token}'` in your "
                "`vocabulary` or use a pretrained `vocabulary` name."
            )

        self.end_token_id = self.token_to_id(end_token)
        # GPT2 uses the same start and pad token as end token, i.e.,
        # "<|endoftext|>".
        self.start_token_id = self.end_token_id
        self.pad_token_id = self.end_token_id

    @classproperty
    def presets(cls):
        return copy.deepcopy(backbone_presets)



# Copyright 2023 The KerasNLP Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import shutil

import numpy as np
import tensorflow as tf
import transformers
from absl import app
from absl import flags

import keras_nlp
from tools.checkpoint_conversion.checkpoint_conversion_utils import (
    get_md5_checksum,
)

PRESET_MAP = {
    "f_net_base_en": "google/fnet-base",
    "f_net_large_en": "google/fnet-large",
}

FLAGS = flags.FLAGS
flags.DEFINE_string(
    "preset", None, f'Must be one of {",".join(PRESET_MAP.keys())}'
)


def convert_checkpoints(hf_model):
    print("\n-> Convert original weights to KerasNLP format.")

    print("\n-> Load KerasNLP model.")
    keras_nlp_model = keras_nlp.models.FNetBackbone.from_preset(
        FLAGS.preset, load_weights=False
    )

    hf_wts = hf_model.state_dict()
    print("Original weights:")
    print(list(hf_wts.keys()))

    keras_nlp_model.get_layer("token_embedding").embeddings.assign(
        hf_wts["embeddings.word_embeddings.weight"]
    )
    keras_nlp_model.get_layer("position_embedding").position_embeddings.assign(
        hf_wts["embeddings.position_embeddings.weight"]
    )
    keras_nlp_model.get_layer("segment_embedding").embeddings.assign(
        hf_wts["embeddings.token_type_embeddings.weight"]
    )

    keras_nlp_model.get_layer("embeddings_layer_norm").gamma.assign(
        hf_wts["embeddings.LayerNorm.weight"]
    )
    keras_nlp_model.get_layer("embeddings_layer_norm").beta.assign(
        hf_wts["embeddings.LayerNorm.bias"]
    )

    keras_nlp_model.get_layer("embedding_projection").kernel.assign(
        hf_wts["embeddings.projection.weight"].T
    )
    keras_nlp_model.get_layer("embedding_projection").bias.assign(
        hf_wts["embeddings.projection.bias"]
    )

    for i in range(keras_nlp_model.num_layers):
        keras_nlp_model.get_layer(
            f"f_net_layer_{i}"
        )._mixing_layer_norm.gamma.assign(
            hf_wts[f"encoder.layer.{i}.fourier.output.LayerNorm.weight"].numpy()
        )
        keras_nlp_model.get_layer(
            f"f_net_layer_{i}"
        )._mixing_layer_norm.beta.assign(
            hf_wts[f"encoder.layer.{i}.fourier.output.LayerNorm.bias"].numpy()
        )

        keras_nlp_model.get_layer(
            f"f_net_layer_{i}"
        )._intermediate_dense.kernel.assign(
            hf_wts[f"encoder.layer.{i}.intermediate.dense.weight"]
            .transpose(1, 0)
            .numpy()
        )
        keras_nlp_model.get_layer(
            f"f_net_layer_{i}"
        )._intermediate_dense.bias.assign(
            hf_wts[f"encoder.layer.{i}.intermediate.dense.bias"].numpy()
        )

        keras_nlp_model.get_layer(
            f"f_net_layer_{i}"
        )._output_dense.kernel.assign(
            hf_wts[f"encoder.layer.{i}.output.dense.weight"]
            .transpose(1, 0)
            .numpy()
        )
        keras_nlp_model.get_layer(f"f_net_layer_{i}")._output_dense.bias.assign(
            hf_wts[f"encoder.layer.{i}.output.dense.bias"].numpy()
        )

        keras_nlp_model.get_layer(
            f"f_net_layer_{i}"
        )._output_layer_norm.gamma.assign(
            hf_wts[f"encoder.layer.{i}.output.LayerNorm.weight"].numpy()
        )
        keras_nlp_model.get_layer(
            f"f_net_layer_{i}"
        )._output_layer_norm.beta.assign(
            hf_wts[f"encoder.layer.{i}.output.LayerNorm.bias"].numpy()
        )

    keras_nlp_model.get_layer("pooled_dense").kernel.assign(
        hf_wts["pooler.dense.weight"].transpose(1, 0).numpy()
    )
    keras_nlp_model.get_layer("pooled_dense").bias.assign(
        hf_wts["pooler.dense.bias"].numpy()
    )

    # Save the model.
    print("\n-> Save KerasNLP model weights.")
    keras_nlp_model.save_weights(os.path.join(FLAGS.preset, "model.h5"))

    return keras_nlp_model


def extract_vocab(hf_tokenizer):
    spm_path = os.path.join(FLAGS.preset, "spiece.model")
    print(f"\n-> Save KerasNLP SPM vocabulary file to `{spm_path}`.")

    shutil.copyfile(
        transformers.utils.hub.get_file_from_repo(
            hf_tokenizer.name_or_path, "spiece.model"
        ),
        spm_path,
    )

    keras_nlp_tokenizer = keras_nlp.models.FNetTokenizer(
        proto=spm_path,
    )
    keras_nlp_preprocessor = keras_nlp.models.FNetPreprocessor(
        keras_nlp_tokenizer
    )

    print("-> Print MD5 checksum of the vocab files.")
    print(f"`{spm_path}` md5sum: ", get_md5_checksum(spm_path))

    return keras_nlp_preprocessor


def check_output(
    keras_nlp_preprocessor,
    keras_nlp_model,
    hf_tokenizer,
    hf_model,
):
    print("\n-> Check the outputs.")
    sample_text = ["cricket is awesome, easily the best sport in the world!"]

    # KerasNLP
    keras_nlp_inputs = keras_nlp_preprocessor(tf.constant(sample_text))
    keras_nlp_output = keras_nlp_model.predict(keras_nlp_inputs)[
        "sequence_output"
    ]

    # HF
    hf_inputs = hf_tokenizer(
        sample_text, padding="max_length", return_tensors="pt"
    )
    hf_output = hf_model(**hf_inputs).last_hidden_state

    print("KerasNLP output:", keras_nlp_output[0, 0, :10])
    print("HF output:", hf_output[0, 0, :10])
    print("Difference:", np.mean(keras_nlp_output - hf_output.detach().numpy()))

    # Show the MD5 checksum of the model weights.
    print(
        "Model md5sum: ",
        get_md5_checksum(os.path.join(FLAGS.preset, "model.h5")),
    )


def main(_):
    os.makedirs(FLAGS.preset)

    hf_model_name = PRESET_MAP[FLAGS.preset]

    print("\n-> Load HF model and HF tokenizer.")
    hf_model = transformers.AutoModel.from_pretrained(hf_model_name)
    hf_model.eval()
    hf_tokenizer = transformers.AutoTokenizer.from_pretrained(hf_model_name)

    keras_nlp_model = convert_checkpoints(hf_model)
    print("\n -> Load KerasNLP preprocessor.")
    keras_nlp_preprocessor = extract_vocab(hf_tokenizer)

    check_output(
        keras_nlp_preprocessor,
        keras_nlp_model,
        hf_tokenizer,
        hf_model,
    )


if __name__ == "__main__":
    flags.mark_flag_as_required("preset")
    app.run(main)



# Copyright 2023 The KerasNLP Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Small utility script to count parameters in our preset checkpoints.

Usage:
python tools/count_preset_params.py
python tools/count_preset_params.py --model BertBackbone
python tools/count_preset_params.py --preset bert_base_multi
"""

import inspect

from absl import app
from absl import flags
from keras.utils.layer_utils import count_params
from tensorflow import keras

import keras_nlp

FLAGS = flags.FLAGS
flags.DEFINE_string("model", None, "The name of a model, e.g. BertBackbone.")
flags.DEFINE_string(
    "preset", None, "The name of a preset, e.g. bert_base_multi."
)


def main(_):
    for name, symbol in keras_nlp.models.__dict__.items():
        if inspect.isclass(symbol) and issubclass(symbol, keras.Model):
            if FLAGS.model and name != FLAGS.model:
                continue
            if not hasattr(symbol, "from_preset"):
                continue
            for preset in symbol.presets:
                if FLAGS.preset and preset != FLAGS.preset:
                    continue
                model = symbol.from_preset(preset)
                params = count_params(model.weights)
                print(f"{name} {preset} {params}")


if __name__ == "__main__":
    app.run(main)



# Copyright 2023 The KerasNLP Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Base sampler class."""

import tensorflow as tf
from tensorflow import keras

from keras_nlp.utils.python_utils import format_docstring

base_sampler_args_docstring = """
    jit_compile: bool, defaults to True. If True, XLA compilation will be used.
    run_eagerly: bool, defaults to False. If True, the sampler will run in
        the eager mode.
    """

call_args_docstring = """
    prompt: a list of integers or an integer Tensor, can be 1D or 2D. The
        initial tokens to append generated tokens.
    token_probability_fn: a function that generates the probability of
        the next token over the whole vocabulary for each input token.
    max_length: int. The max length of generated sequence.
    mask: a tensor, defaults to None. The padding mask of the prompt.
    end_token_id: int, defaults to None. The token marking the end of the
        sequence, once encountered the generation is finished for the exact
        sequence. If None, every sequence is generated up to `max_length`.
        If set, all tokens after encountering `end_token_id` will be
        replaced with `pad_token_id`.
    from_logits: bool, defaults to True. Indicate if the `token_probability_fn`
        returns logits. If False, `token_probability_fn` returns probability
        distributions.
    """


@format_docstring(
    base_sampler_args=base_sampler_args_docstring, call_args=call_args_docstring
)
@keras.utils.register_keras_serializable(package="keras_nlp")
class Sampler:
    """Base sampler class.

    Args:
        {{base_sampler_args}}

    Call Args:
        {{call_args}}

    The inputs and outputs of Sampler class are both token ids.

    Subclassers should always implement the `get_next_token()` method, which
    gets the next token based on probability distribution over vocab tokens.
    Please check available subclass samplers for examples. If you need more
    control over the sampling process, please implement `sample()` method
    instead, see `keras_nlp.samplers.BeamSampler` for examples.

    Examples:

    Basic usage:
    ```python
    VOCAB_SIZE = 10

    # Create a dummy model to predict the next token. Note that the output is
    # random without training, here we jsut demo how `samplers` works.
    model = keras.Sequential(
        [
            keras.Input(shape=[None]),
            keras.layers.Embedding(
                input_dim=VOCAB_SIZE,
                output_dim=16,
            ),
            keras.layers.Dense(VOCAB_SIZE, activation="softmax"),
        ]
    )

    # Define a function that outputs the next token's probability for each token
    # in the input sequence.
    def token_probability_fn(inputs, mask):
        return model(inputs)

    prompt = tf.fill((8, 1), 1)

    sampler = keras_nlp.samplers.GreedySampler()
    # Print the generated sequence (token ids).
    print(sampler(prompt, token_probability_fn, max_length=10, end_token_id=2))
    ```

    Use with string inputs:
    ```python
    vocab = ["[UNK]", "[PAD]", "[END]", "the", "quick", "brown", "fox"]
    tokenizer = keras_nlp.tokenizers.WordPieceTokenizer(
        vocabulary=vocab,
        lowercase=True,
    )
    FEATURE_SIZE = 16
    VOCAB_SIZE = len(vocab)
    # Create a dummy model to predict the next token.
    model = keras.Sequential(
        [
            keras.Input(shape=[None]),
            keras.layers.Embedding(
                input_dim=VOCAB_SIZE,
                output_dim=FEATURE_SIZE,
            ),
            keras.layers.Dense(VOCAB_SIZE, activation="softmax"),
        ]
    )
    # Define a function that outputs the next token's probability for each token
    # in the input sequence.
    def token_probability_fn(inputs, mask):
        return model(inputs)

    prompt = tokenizer("the quick brown fox")
    sampler = keras_nlp.samplers.GreedySampler()
    generated = sampler(
        prompt,
        token_probability_fn,
        max_length=10,
        end_token_id=tokenizer.token_to_id("[END]")
    )
    print(tokenizer.detokenize(generated))
    ```
    """

    def __init__(
        self,
        jit_compile=True,
        run_eagerly=False,
    ):
        if run_eagerly and jit_compile:
            raise ValueError(
                "XLA cannot be turned on under eager mode, received "
                "`jit_compile=True` and `run_eagerly=True`. Please either set "
                "`jit_compile=False` or set `run_eagerly=False`."
            )
        self.jit_compile = jit_compile
        self.run_eagerly = run_eagerly

    def _validate_prompt_and_mask(self, prompt, mask):
        """Helper method to validate input prompt."""
        if not isinstance(prompt, (list, tf.RaggedTensor, tf.Tensor)):
            raise ValueError(
                "`prompt` must be one of `list`, `tf.RaggedTensor` or "
                f"`tf.Tensor`, but received: prompt={type(prompt)}."
            )

        if isinstance(prompt, tf.RaggedTensor):
            if mask:
                raise ValueError(
                    "`mask` is only valid when `prompt` is a list or dense "
                    f"tensor, but received type(prompt)={type(prompt)}."
                )
            return prompt, mask

        if isinstance(prompt, list):
            prompt = tf.convert_to_tensor(prompt)
        if not mask:
            mask = tf.cast(tf.ones_like(prompt), dtype=tf.bool)
        prompt = tf.ragged.boolean_mask(prompt, mask)
        return prompt, mask

    def _validate_token_probability_fn(
        self, token_probability_fn, prompt, mask
    ):
        """Helper method to validate `token_probability_fn` output."""
        test_pred = token_probability_fn(prompt, mask=mask)
        if len(test_pred.shape) != 3:
            raise ValueError(
                "Output of `token_probability_fn` is not a 3D tensor, "
                "please provide a function with the output shape "
                "[batch_size, sequence_length, vocab_size]."
            )

    def _pad_prompt(self, prompt, max_length):
        """Pad prompt to `max_length`."""
        mask = tf.ones_like(prompt, dtype=tf.bool)
        mask = mask.to_tensor(shape=(None, max_length))
        prompt = prompt.to_tensor(shape=(None, max_length))
        return prompt, mask

    def _mask_tokens_after_end_token(
        self,
        generated_result,
        original_padding_mask,
        max_length,
        end_token_id,
    ):
        """Helper function to truncate the tokens after the end token."""
        # Create a tensor with True for each end_token_id.
        end_tokens = generated_result == end_token_id
        # Remove all end_token_ids in the original input.
        end_tokens = end_tokens & (original_padding_mask == tf.constant(False))
        # Find index of first end_token_id.
        end_indices = tf.math.argmax(end_tokens, -1)
        # Use max_length if no `end_token_id` is found.
        end_indices = tf.where(
            end_indices == 0,
            tf.cast(max_length, dtype=end_indices.dtype),
            end_indices,
        )
        # Truncate out tokens after (including) the end token.
        mask_indices = tf.sequence_mask(end_indices, maxlen=max_length)
        return tf.ragged.boolean_mask(generated_result, mask_indices)

    def __call__(
        self,
        prompt,
        token_probability_fn,
        max_length,
        mask=None,
        end_token_id=None,
        from_logits=True,
    ):
        prompt, mask = self._validate_prompt_and_mask(prompt, mask)
        input_is_1d = prompt.shape.rank == 1
        if input_is_1d:
            prompt = tf.RaggedTensor.from_tensor(prompt[tf.newaxis, :])

        shortest_prompt_len = tf.reduce_min(prompt.row_lengths())
        # Pad prompt to be a dense Tensor of shape [batch_size, max_length].
        # This step is required for XLA compatibility because XLA requires a
        # static shape, which means we cannot concatenate generated token to
        # current prompt.
        prompt, mask = self._pad_prompt(prompt, max_length)
        original_padding_mask = tf.identity(mask)
        self._validate_token_probability_fn(token_probability_fn, prompt, mask)

        # Convert `sample` method to a `tf.function` if `self.run_eagerly=False`
        # , and turn on `jit_compile` accordingly.
        sample = self.sample
        if not self.run_eagerly:
            sample = tf.function(self.sample, jit_compile=self.jit_compile)
        prompt = sample(
            prompt,
            token_probability_fn,
            mask,
            max_length - shortest_prompt_len,
            from_logits,
        )

        # Mask out tokens after `end_token_id`.
        if end_token_id is not None:
            prompt = self._mask_tokens_after_end_token(
                prompt,
                original_padding_mask,
                max_length,
                end_token_id,
            )

        return tf.squeeze(prompt, axis=0) if input_is_1d else prompt

    def get_next_token(self, next_token_probs):
        """Get the next token.

        Args:
            next_token_probs: a Tensor, the probability distribution for next
                token over all vocab tokens.

        Get the next token based on given probability distribution over tokens.
        Subclasses must implement this method.
        """
        raise NotImplementedError

    def sample(
        self, prompt, token_probability_fn, mask, num_steps, from_logits=True
    ):
        """Sampling logic implementation.

        Args:
            prompt: a dense int Tensor of shape [batch_size, max_length]. The
                placeholder for generated sequence.
            token_probability_fn: a function that generates the probability of
                the next token over the whole vocabulary for each input token.
            mask: a dense bool Tensor of shape [batch_size, max_length]. The
                mask of prompt.
            num_steps: int. The remaining number of tokens to generate.
            from_logits: bool, defaults to True. Indicate if the
                `token_probability_fn` returns logits. If False,
                `token_probability_fn` returns probability distributions.

        Returns:
            A dense int Tensor, representing the generated text in token id
            space.
        """
        batch_size, max_length = tf.shape(prompt)[0], tf.shape(prompt)[1]
        max_length = tf.cast(max_length, num_steps.dtype)
        # The index of the last non-padding token in prompt. Since all sequences
        # are aligned to the right side, the index is the same for all.
        current_index = max_length - num_steps

        def one_step(current_index, prompt, mask):
            probs = token_probability_fn(prompt, mask)
            next_token_probs = tf.gather(
                probs,
                tf.repeat(current_index - 1, batch_size),
                axis=1,
                batch_dims=1,
            )
            if from_logits:
                next_token_probs = keras.activations.softmax(
                    next_token_probs, axis=-1
                )
            next_token = self.get_next_token(next_token_probs)
            next_token = tf.cast(next_token, prompt.dtype)
            next_token = tf.where(
                mask[:, current_index], prompt[:, current_index], next_token
            )
            mask = tf.tensor_scatter_nd_update(
                tensor=mask,
                indices=tf.stack(
                    (
                        tf.cast(
                            tf.range(batch_size), dtype=current_index.dtype
                        ),
                        tf.repeat(current_index, batch_size),
                    ),
                    axis=1,
                ),
                updates=tf.repeat(True, batch_size),
            )

            # Append the next token to current sequence.
            prompt = tf.tensor_scatter_nd_update(
                tensor=prompt,
                indices=tf.stack(
                    (
                        tf.cast(
                            tf.range(batch_size), dtype=current_index.dtype
                        ),
                        tf.repeat(current_index, batch_size),
                    ),
                    axis=1,
                ),
                updates=next_token,
            )

            current_index = tf.add(current_index, 1)
            return (current_index, prompt, mask)

        # Run a while loop till `max_length` of tokens has been generated.
        current_index, prompt, mask = tf.while_loop(
            cond=lambda current_index, prompt, mask: tf.less(
                current_index, max_length
            ),
            body=one_step,
            loop_vars=(current_index, prompt, mask),
        )
        return prompt

    def get_config(self):
        return {
            "jit_compile": self.jit_compile,
        }



# Copyright 2023 The KerasNLP Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import io
import os

import sentencepiece
import tensorflow as tf
from absl.testing import parameterized
from tensorflow import keras

from keras_nlp.tokenizers.sentence_piece_tokenizer import SentencePieceTokenizer


class SentencePieceTokenizerTest(tf.test.TestCase, parameterized.TestCase):
    def setUp(self):
        super().setUp()
        bytes_io = io.BytesIO()
        vocab_data = tf.data.Dataset.from_tensor_slices(
            ["the quick brown fox."]
        )
        sentencepiece.SentencePieceTrainer.train(
            sentence_iterator=vocab_data.as_numpy_iterator(),
            model_writer=bytes_io,
            vocab_size=7,
            model_type="WORD",
        )
        self.proto = bytes_io.getvalue()

    def test_tokenize(self):
        input_data = ["the quick brown fox."]
        tokenizer = SentencePieceTokenizer(
            proto=self.proto,
        )
        call_output = tokenizer(input_data)
        tokenize_output = tokenizer.tokenize(input_data)
        self.assertIsInstance(call_output, tf.RaggedTensor)
        self.assertAllEqual(call_output, [[6, 5, 3, 4]])
        self.assertAllEqual(tokenize_output, [[6, 5, 3, 4]])

    def test_scalar_tokenize(self):
        input_data = "the quick brown fox."
        tokenizer = SentencePieceTokenizer(
            proto=self.proto,
        )
        call_output = tokenizer(input_data)
        tokenize_output = tokenizer.tokenize(input_data)
        self.assertIsInstance(call_output, tf.Tensor)
        self.assertAllEqual(call_output, [6, 5, 3, 4])
        self.assertAllEqual(tokenize_output, [6, 5, 3, 4])

    def test_dense_output(self):
        input_data = ["the quick brown fox."]
        tokenizer = SentencePieceTokenizer(
            proto=self.proto,
            sequence_length=10,
        )
        output_data = tokenizer(input_data)
        self.assertIsInstance(output_data, tf.Tensor)
        self.assertAllEqual(output_data, [[6, 5, 3, 4, 0, 0, 0, 0, 0, 0]])

    def test_string_tokenize(self):
        input_data = ["the quick brown fox."]
        tokenizer = SentencePieceTokenizer(
            proto=self.proto,
            dtype="string",
        )
        output_data = tokenizer(input_data)
        self.assertAllEqual(
            output_data,
            tf.ragged.constant([["▁the", "▁quick", "▁brown", "▁fox."]]),
        )

    def test_detokenize(self):
        input_data = [[6, 5, 3, 4]]
        tokenizer = SentencePieceTokenizer(
            proto=self.proto,
        )
        output_data = tokenizer.detokenize(input_data)
        self.assertAllEqual(output_data, ["the quick brown fox."])

    def test_accessors(self):
        tokenizer = SentencePieceTokenizer(
            proto=self.proto,
        )
        self.assertEqual(
            tokenizer.get_vocabulary(),
            ["<unk>", "<s>", "</s>", "▁brown", "▁fox.", "▁quick", "▁the"],
        )
        self.assertEqual(type(tokenizer.get_vocabulary()), list)
        self.assertEqual(tokenizer.vocabulary_size(), 7)
        self.assertEqual(type(tokenizer.vocabulary_size()), int)
        self.assertEqual(tokenizer.id_to_token(0), "<unk>")
        self.assertEqual(tokenizer.id_to_token(5), "▁quick")
        self.assertEqual(type(tokenizer.id_to_token(0)), str)
        self.assertEqual(tokenizer.token_to_id("<unk>"), 0)
        self.assertEqual(tokenizer.token_to_id("▁quick"), 5)
        self.assertEqual(type(tokenizer.token_to_id("<unk>")), int)

    def test_functional_model(self):
        input_data = tf.constant(["the quick brown fox."])
        tokenizer = SentencePieceTokenizer(
            proto=self.proto,
        )
        inputs = keras.Input(dtype="string", shape=())
        outputs = tokenizer.detokenize(tokenizer.tokenize(inputs))
        model = keras.Model(inputs, outputs)
        model_output = model(input_data)
        self.assertAllEqual(model_output, ["the quick brown fox."])

    def test_from_file(self):
        filepath = os.path.join(self.get_temp_dir(), "model.txt")
        input_data = ["the quick brown fox."]
        with tf.io.gfile.GFile(filepath, "wb") as file:
            file.write(self.proto)
        tokenizer = SentencePieceTokenizer(
            proto=filepath,
        )
        output_data = tokenizer(input_data)
        self.assertAllEqual(output_data, [[6, 5, 3, 4]])

    def test_tokenize_then_batch(self):
        tokenizer = SentencePieceTokenizer(
            proto=self.proto,
        )

        ds = tf.data.Dataset.from_tensor_slices(
            ["the quick brown fox.", "the quick", "the", "quick brown fox."]
        )
        ds = ds.map(tokenizer).apply(
            tf.data.experimental.dense_to_ragged_batch(4)
        )
        output_data = ds.take(1).get_single_element()

        expected = [
            [6, 5, 3, 4],
            [6, 5],
            [6],
            [5, 3, 4],
        ]
        for i in range(4):
            self.assertAllEqual(output_data[i], expected[i])

    def test_batch_then_tokenize(self):
        tokenizer = SentencePieceTokenizer(
            proto=self.proto,
        )

        ds = tf.data.Dataset.from_tensor_slices(
            ["the quick brown fox.", "the quick", "the", "quick brown fox."]
        )
        ds = ds.batch(4).map(tokenizer)
        output_data = ds.take(1).get_single_element()

        expected = [
            [6, 5, 3, 4],
            [6, 5],
            [6],
            [5, 3, 4],
        ]
        for i in range(4):
            self.assertAllEqual(output_data[i], expected[i])

    def test_config(self):
        input_data = ["the quick brown whale."]
        original_tokenizer = SentencePieceTokenizer(
            proto=self.proto,
        )
        cloned_tokenizer = SentencePieceTokenizer.from_config(
            original_tokenizer.get_config()
        )
        self.assertAllEqual(
            original_tokenizer(input_data),
            cloned_tokenizer(input_data),
        )

    @parameterized.named_parameters(
        ("tf_format", "tf", "model"),
        ("keras_format", "keras_v3", "model.keras"),
    )
    def test_saved_model(self, save_format, filename):
        filepath = os.path.join(self.get_temp_dir(), "model.txt")
        input_data = tf.constant(["the quick brown whale."])
        with tf.io.gfile.GFile(filepath, "wb") as file:
            file.write(self.proto)
        tokenizer = SentencePieceTokenizer(
            proto=filepath,
        )
        inputs = keras.Input(dtype="string", shape=())
        outputs = tokenizer(inputs)
        model = keras.Model(inputs, outputs)
        path = os.path.join(self.get_temp_dir(), filename)
        model.save(path, save_format=save_format)
        restored_model = keras.models.load_model(path)
        self.assertAllEqual(
            model(input_data),
            restored_model(input_data),
        )



# Copyright 2023 The KerasNLP Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for Text Generation Utils."""

import random

import numpy as np
import tensorflow as tf
from absl.testing import parameterized
from tensorflow import keras

from keras_nlp.utils.text_generation import beam_search
from keras_nlp.utils.text_generation import greedy_search
from keras_nlp.utils.text_generation import random_search
from keras_nlp.utils.text_generation import top_k_search
from keras_nlp.utils.text_generation import top_p_search


class GreedySearchTextGenerationTest(tf.test.TestCase, parameterized.TestCase):
    def setUp(self):
        super().setUp()
        self.vocab_size = 10
        self.feature_size = 16

        # Create a dummy model to predict the next token.
        model = keras.Sequential(
            [
                keras.Input(shape=[None]),
                keras.layers.Embedding(
                    input_dim=self.vocab_size,
                    output_dim=self.feature_size,
                ),
                keras.layers.Dense(self.vocab_size),
                keras.layers.Softmax(),
            ]
        )

        def token_probability_fn(inputs):
            return model(inputs)[:, -1, :]

        self.token_probability_fn = token_probability_fn

    def test_generate_with_1d_prompt(self):
        inputs = tf.constant([1])
        outputs = greedy_search(self.token_probability_fn, inputs, max_length=5)
        self.assertEqual(outputs.shape, [5])

    def test_generate_with_2d_prompt(self):
        inputs = tf.constant([[1], [1]])
        outputs = greedy_search(self.token_probability_fn, inputs, max_length=5)
        self.assertEqual(outputs.shape, [2, 5])

    def test_generate_with_list_prompt(self):
        inputs = [[1], [1]]
        outputs = greedy_search(self.token_probability_fn, inputs, max_length=5)
        self.assertEqual(outputs.shape, [2, 5])

    def test_generate_with_ragged_prompt(self):
        def token_probability_fn(inputs):
            # Assert that user function is passed only dense tensors.
            self.assertIsInstance(inputs, tf.Tensor)
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, 2, axis=0)

        inputs = tf.ragged.constant([[1], [2, 1, 2]])
        outputs = greedy_search(token_probability_fn, inputs, max_length=5)
        self.assertEqual(outputs.shape, [2, 5])

    def test_assert_generation_is_correct(self):
        def token_probability_fn(inputs):
            batch_size = inputs.shape[0]
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, batch_size, axis=0)

        batch_size = 10
        inputs = 3 * tf.ones([batch_size, 1], dtype=tf.int32)
        max_length = 3
        outputs = greedy_search(
            token_probability_fn, inputs, max_length=max_length
        )
        self.assertAllEqual(
            outputs, 3 * tf.ones(shape=[batch_size, max_length])
        )

    def test_end_token_id(self):
        def token_probability_fn(inputs):
            batch_size = inputs.shape[0]
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, batch_size, axis=0)

        max_length = 5
        inputs = tf.constant([[0, 1], [1, 2]])
        outputs = greedy_search(
            token_probability_fn,
            inputs,
            max_length=max_length,
            end_token_id=2,
            pad_token_id=0,
        )
        expected_outputs = tf.tile([[3], [0]], [1, max_length - 2])
        expected_outputs = tf.concat([inputs, expected_outputs], axis=1)
        self.assertAllEqual(outputs, expected_outputs)

    @parameterized.named_parameters(
        ("dense_jit_compile_false", False, False),
        ("dense_jit_compile_true", True, False),
        ("ragged_jit_compile_false", False, True),
    )
    def test_model_compile(self, jit_compile, ragged):
        def token_probability_fn(inputs):
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, 2, axis=0)

        max_length = 5

        class TestModel(tf.keras.Model):
            def call(self, inputs):
                generated = greedy_search(
                    token_probability_fn,
                    inputs,
                    max_length=max_length,
                    end_token_id=2,
                    pad_token_id=0,
                )
                return generated

        if ragged:
            inputs = tf.ragged.constant([[0, 1], [1, 1, 2]])
            expected_outputs = tf.constant([[0, 1, 3, 3, 3], [1, 1, 2, 0, 0]])
        else:
            inputs = tf.constant([[0, 1], [1, 2]])
            expected_outputs = [[0, 1, 3, 3, 3], [1, 2, 0, 0, 0]]

        model = TestModel()
        model.compile(jit_compile=jit_compile)

        outputs = model.predict(inputs)
        self.assertAllEqual(outputs, expected_outputs)

    @parameterized.named_parameters(
        ("dense_jit_compile_false", False, False),
        ("dense_jit_compile_true", True, False),
        ("ragged_jit_compile_false", False, True),
    )
    def test_model_compile_batched_ds(self, jit_compile, ragged):
        def token_probability_fn(inputs):
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, 2, axis=0)

        max_length = 5

        class TestModel(tf.keras.Model):
            def call(self, inputs):
                generated = greedy_search(
                    token_probability_fn,
                    inputs,
                    max_length=max_length,
                    end_token_id=2,
                    pad_token_id=0,
                )
                return generated

        if ragged:
            inputs = tf.ragged.constant([[0, 1], [1, 1, 2]])
            expected_outputs = tf.constant([[0, 1, 3, 3, 3], [1, 1, 2, 0, 0]])
        else:
            inputs = tf.constant([[0, 1], [1, 2]])
            expected_outputs = [[0, 1, 3, 3, 3], [1, 2, 0, 0, 0]]

        ds = tf.data.Dataset.from_tensor_slices(inputs).batch(2)

        model = TestModel()
        model.compile(jit_compile=jit_compile)

        outputs = model.predict(ds)
        self.assertAllEqual(outputs, expected_outputs)


class BeamSearchTextGenerationTest(tf.test.TestCase, parameterized.TestCase):
    def setUp(self):
        super().setUp()
        self.vocab_size = 10
        self.feature_size = 16

        # Create a dummy model to predict the next token.
        model = keras.Sequential(
            [
                keras.Input(shape=[None]),
                keras.layers.Embedding(
                    input_dim=self.vocab_size,
                    output_dim=self.feature_size,
                ),
                keras.layers.Dense(self.vocab_size),
                keras.layers.Softmax(),
            ]
        )

        def token_probability_fn(inputs):
            return model(inputs)[:, -1, :]

        self.token_probability_fn = token_probability_fn

    def test_generate_with_1d_prompt(self):
        inputs = tf.constant([1])
        outputs = beam_search(
            self.token_probability_fn,
            inputs,
            max_length=5,
            num_beams=5,
        )
        self.assertEqual(outputs.shape, [5])

    def test_generate_with_max_length_prompt(self):
        inputs = tf.ones(shape=(5,))
        outputs = beam_search(
            self.token_probability_fn,
            inputs,
            max_length=5,
            num_beams=5,
        )
        self.assertEqual(outputs.shape, [5])

        inputs = tf.ones(shape=(6,))
        outputs = beam_search(
            self.token_probability_fn,
            inputs,
            max_length=5,
            num_beams=5,
        )
        self.assertEqual(outputs.shape, [6])

    def test_generate_with_2d_prompt(self):
        inputs = tf.constant([[1], [1]])
        outputs = beam_search(
            self.token_probability_fn,
            inputs,
            max_length=5,
            num_beams=5,
        )
        self.assertEqual(outputs.shape, [2, 5])

    def test_generate_with_list_prompt(self):
        inputs = [[1], [1]]
        outputs = beam_search(
            self.token_probability_fn,
            inputs,
            max_length=5,
            num_beams=5,
        )
        self.assertEqual(outputs.shape, [2, 5])

    def test_generate_with_ragged_prompt(self):
        def token_probability_fn(inputs):
            # Assert that user function is passed only dense tensors.
            self.assertIsInstance(inputs, tf.Tensor)
            repeat_times = tf.shape(inputs)[0]
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, repeat_times, axis=0)

        inputs = tf.ragged.constant([[1], [2, 1, 2]])
        outputs = beam_search(
            token_probability_fn, inputs, max_length=5, num_beams=2
        )
        self.assertEqual(outputs.shape, [2, 5])

    def test_one_beam_generation(self):
        for i in range(50):
            inputs = tf.constant([random.randint(0, 9)])
            beam_output = beam_search(
                self.token_probability_fn,
                inputs,
                max_length=5,
                num_beams=1,
            )
            greedy_output = greedy_search(
                self.token_probability_fn,
                inputs,
                max_length=5,
            )
            self.assertAllEqual(beam_output, greedy_output)

    def test_assert_generation_is_correct(self):
        def token_probability_fn(inputs):
            batch_size = inputs.shape[0]
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, batch_size, axis=0)

        batch_size = 10
        inputs = 3 * tf.ones([batch_size, 1], dtype=tf.int32)
        max_length = 3
        for i in range(1, 10):
            outputs = beam_search(
                token_probability_fn,
                inputs,
                max_length=max_length,
                num_beams=i,
            )
            self.assertAllEqual(
                outputs, 3 * tf.ones(shape=[batch_size, max_length])
            )

    def test_end_token_id(self):
        def token_probability_fn(inputs):
            batch_size = inputs.shape[0]
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, batch_size, axis=0)

        max_length = 5
        inputs = tf.constant([[0, 1], [1, 2]])
        outputs = beam_search(
            token_probability_fn,
            inputs,
            max_length=max_length,
            num_beams=2,
            end_token_id=2,
            pad_token_id=0,
        )
        expected_outputs = tf.tile([[3], [0]], [1, max_length - 2])
        expected_outputs = tf.concat([inputs, expected_outputs], axis=1)
        self.assertAllEqual(outputs, expected_outputs)

    @parameterized.named_parameters(
        ("dense_jit_compile_false", False, False),
        ("dense_jit_compile_true", True, False),
        ("ragged_jit_compile_false", False, True),
    )
    def test_model_compile(self, jit_compile, ragged):
        def token_probability_fn(inputs):
            repeat_times = tf.shape(inputs)[0]
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, repeat_times, axis=0)

        max_length = 5

        class TestModel(tf.keras.Model):
            def call(self, inputs):
                generated = beam_search(
                    token_probability_fn,
                    inputs,
                    max_length=max_length,
                    num_beams=2,
                    end_token_id=2,
                    pad_token_id=0,
                )
                return generated

        if ragged:
            inputs = tf.ragged.constant([[0, 1], [1, 1, 2]])
            expected_outputs = tf.constant([[0, 1, 3, 3, 3], [1, 1, 2, 0, 0]])
        else:
            inputs = tf.constant([[0, 1], [1, 2]])
            expected_outputs = [[0, 1, 3, 3, 3], [1, 2, 0, 0, 0]]

        model = TestModel()
        model.compile(jit_compile=jit_compile)

        outputs = model.predict(inputs)
        self.assertAllEqual(outputs, expected_outputs)

    @parameterized.named_parameters(
        ("dense_jit_compile_false", False, False),
        ("dense_jit_compile_true", True, False),
        ("ragged_jit_compile_false", False, True),
    )
    def test_model_compile_batched_ds(self, jit_compile, ragged):
        def token_probability_fn(inputs):
            repeat_times = tf.shape(inputs)[0]
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, repeat_times, axis=0)

        max_length = 5

        class TestModel(tf.keras.Model):
            def call(self, inputs):
                generated = beam_search(
                    token_probability_fn,
                    inputs,
                    max_length=max_length,
                    num_beams=2,
                    end_token_id=2,
                    pad_token_id=0,
                )
                return generated

        if ragged:
            inputs = tf.ragged.constant([[0, 1], [1, 1, 2]])
            expected_outputs = tf.constant([[0, 1, 3, 3, 3], [1, 1, 2, 0, 0]])
        else:
            inputs = tf.constant([[0, 1], [1, 2]])
            expected_outputs = [[0, 1, 3, 3, 3], [1, 2, 0, 0, 0]]

        ds = tf.data.Dataset.from_tensor_slices(inputs).batch(2)

        model = TestModel()
        model.compile(jit_compile=jit_compile)

        outputs = model.predict(ds)
        self.assertAllEqual(outputs, expected_outputs)


class RandomSearchTextGenerationTest(tf.test.TestCase, parameterized.TestCase):
    def setUp(self):
        super().setUp()
        self.vocab_size = 10
        self.feature_size = 16

        # Create a dummy model to predict the next token.
        model = keras.Sequential(
            [
                keras.Input(shape=[None]),
                keras.layers.Embedding(
                    input_dim=self.vocab_size,
                    output_dim=self.feature_size,
                ),
                keras.layers.Dense(self.vocab_size),
                keras.layers.Softmax(),
            ]
        )

        def token_probability_fn(inputs):
            return model(inputs)[:, -1, :]

        self.token_probability_fn = token_probability_fn

    def test_generate_with_1d_prompt(self):
        inputs = tf.constant([1])
        outputs = random_search(self.token_probability_fn, inputs, max_length=5)
        self.assertEqual(outputs.shape, [5])

    def test_generate_with_2d_prompt(self):
        inputs = tf.constant([[1], [1]])
        outputs = random_search(self.token_probability_fn, inputs, max_length=5)
        self.assertEqual(outputs.shape, [2, 5])

    def test_generate_with_list_prompt(self):
        inputs = [[1], [1]]
        outputs = random_search(self.token_probability_fn, inputs, max_length=5)
        self.assertEqual(outputs.shape, [2, 5])

    def test_generate_with_ragged_prompt(self):
        def token_probability_fn(inputs):
            # Assert that user function is passed only dense tensors.
            self.assertIsInstance(inputs, tf.Tensor)
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, 2, axis=0)

        inputs = tf.ragged.constant([[1], [2, 1, 2]])
        outputs = random_search(token_probability_fn, inputs, max_length=5)
        self.assertEqual(outputs.shape, [2, 5])

    def test_assert_probability_distribution_generation_is_correct(self):
        def token_probability_fn(inputs):
            batch_size = inputs.shape[0]
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, batch_size, axis=0)

        batch_size = 10
        inputs = 3 * tf.ones([batch_size, 1], dtype=tf.int32)
        max_length = 3

        outputs_count = np.array([0, 0, 0, 0])
        tf.random.set_seed(42)
        for i in range(64):
            outputs = random_search(
                token_probability_fn, inputs, max_length=max_length, seed=42
            )
            flatten_predictions = tf.reshape(outputs[:, 1:], [-1])
            for pred in flatten_predictions:
                outputs_count[pred] += 1
        self.assertAllClose(
            outputs_count / np.sum(outputs_count),
            [0.0, 0.0, 0.0, 1.0],
            rtol=0.2,
        )

    def test_end_token_id(self):
        def token_probability_fn(inputs):
            batch_size = inputs.shape[0]
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, batch_size, axis=0)

        max_length = 5
        inputs = tf.constant([[0, 1], [1, 2]])
        tf.random.set_seed(42)
        outputs = random_search(
            token_probability_fn,
            inputs,
            max_length=max_length,
            seed=42,
            end_token_id=2,
            pad_token_id=0,
        )
        # Random sampling result with seed 42.
        expected_outputs = tf.tile([[3], [0]], [1, max_length - 2])
        expected_outputs = tf.concat([inputs, expected_outputs], axis=1)
        self.assertAllEqual(outputs, expected_outputs)

    @parameterized.named_parameters(
        ("dense_jit_compile_false", False, False),
        ("dense_jit_compile_true", True, False),
        ("ragged_jit_compile_false", False, True),
    )
    def test_model_compile(self, jit_compile, ragged):
        def token_probability_fn(inputs):
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, 2, axis=0)

        max_length = 5

        class TestModel(tf.keras.Model):
            def call(self, inputs):
                generated = random_search(
                    token_probability_fn,
                    inputs,
                    max_length=max_length,
                    seed=42,
                    end_token_id=2,
                    pad_token_id=0,
                )
                return generated

        if ragged:
            inputs = tf.ragged.constant([[0, 1], [1, 1, 2]])
            expected_outputs = tf.constant([[0, 1, 3, 3, 3], [1, 1, 2, 0, 0]])
        else:
            inputs = tf.constant([[0, 1], [1, 2]])
            expected_outputs = [[0, 1, 3, 3, 3], [1, 2, 0, 0, 0]]

        model = TestModel()
        model.compile(jit_compile=jit_compile)

        outputs = model(inputs)
        self.assertAllEqual(outputs, expected_outputs)

    @parameterized.named_parameters(
        ("dense_jit_compile_false", False, False),
        ("dense_jit_compile_true", True, False),
        ("ragged_jit_compile_false", False, True),
    )
    def test_model_compile_batched_ds(self, jit_compile, ragged):
        def token_probability_fn(inputs):
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, 2, axis=0)

        max_length = 5

        class TestModel(tf.keras.Model):
            def call(self, inputs):
                generated = random_search(
                    token_probability_fn,
                    inputs,
                    max_length=max_length,
                    seed=42,
                    end_token_id=2,
                    pad_token_id=0,
                )
                return generated

        if ragged:
            inputs = tf.ragged.constant([[0, 1], [1, 1, 2]])
            expected_outputs = tf.constant([[0, 1, 3, 3, 3], [1, 1, 2, 0, 0]])
        else:
            inputs = tf.constant([[0, 1], [1, 2]])
            expected_outputs = [[0, 1, 3, 3, 3], [1, 2, 0, 0, 0]]

        ds = tf.data.Dataset.from_tensor_slices(inputs).batch(2)

        model = TestModel()
        model.compile(jit_compile=jit_compile)

        outputs = model.predict(ds)
        self.assertAllEqual(outputs, expected_outputs)

    def test_from_logits(self):
        def token_logits_fn(inputs):
            batch_size = inputs.shape[0]
            prob = tf.constant([[1.0, 2.0, 3, 0, 4.0]])
            return tf.repeat(prob, batch_size, axis=0)

        def token_probability_fn(inputs):
            batch_size = inputs.shape[0]
            prob = keras.activations.softmax(
                tf.constant([[1.0, 2.0, 3, 0, 4.0]])
            )
            return tf.repeat(prob, batch_size, axis=0)

        max_length = 5
        inputs = tf.constant([[0, 1], [1, 2]])
        tf.random.set_seed(42)
        output_logit = random_search(
            token_logits_fn,
            inputs,
            max_length=max_length,
            from_logits=True,
            seed=42,
        )
        tf.random.set_seed(42)
        output_probs = random_search(
            token_probability_fn,
            inputs,
            max_length=max_length,
            from_logits=False,
            seed=42,
        )
        self.assertAllEqual(output_logit, output_probs)


class TopKSearchTextGenerationTest(tf.test.TestCase, parameterized.TestCase):
    def setUp(self):
        super().setUp()
        self.vocab_size = 10
        self.feature_size = 16

        # Create a dummy model to predict the next token.
        model = keras.Sequential(
            [
                keras.Input(shape=[None]),
                keras.layers.Embedding(
                    input_dim=self.vocab_size,
                    output_dim=self.feature_size,
                ),
                keras.layers.Dense(self.vocab_size),
                keras.layers.Softmax(),
            ]
        )

        def token_probability_fn(inputs):
            return model(inputs)[:, -1, :]

        self.token_probability_fn = token_probability_fn

    def test_generate_with_1d_prompt(self):
        inputs = tf.constant([1])
        outputs = top_k_search(
            self.token_probability_fn, inputs, max_length=5, k=2
        )
        self.assertEqual(outputs.shape, [5])

    def test_generate_with_k_too_big(self):
        inputs = tf.constant([1])
        outputs = top_k_search(
            self.token_probability_fn,
            inputs,
            max_length=5,
            k=self.vocab_size + 1,
        )
        self.assertEqual(outputs.shape, [5])

    def test_generate_with_2d_prompt(self):
        inputs = tf.constant([[1], [1]])
        outputs = top_k_search(
            self.token_probability_fn, inputs, max_length=5, k=2
        )
        self.assertEqual(outputs.shape, [2, 5])

    def test_generate_with_list_prompt(self):
        inputs = [[1], [1]]
        outputs = top_k_search(
            self.token_probability_fn, inputs, max_length=5, k=2
        )
        self.assertEqual(outputs.shape, [2, 5])

    def test_generate_with_ragged_prompt(self):
        def token_probability_fn(inputs):
            # Assert that user function is passed only dense tensors.
            self.assertIsInstance(inputs, tf.Tensor)
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, 2, axis=0)

        inputs = tf.ragged.constant([[1], [2, 1, 2]])
        outputs = top_k_search(token_probability_fn, inputs, max_length=5, k=2)
        self.assertEqual(outputs.shape, [2, 5])

    def test_assert_probability_distribution_generation_is_correct(self):
        def token_probability_fn(inputs):
            batch_size = inputs.shape[0]
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, batch_size, axis=0)

        batch_size = 10
        inputs = 3 * tf.ones([batch_size, 1], dtype=tf.int32)
        max_length = 3

        outputs_count = np.array([0, 0, 0, 0])
        tf.random.set_seed(42)
        for _ in range(64):
            outputs = top_k_search(
                token_probability_fn,
                inputs,
                max_length=max_length,
                k=2,
                seed=42,
            )
            flatten_predictions = tf.reshape(outputs[:, 1:], [-1])
            for pred in flatten_predictions:
                outputs_count[pred] += 1
        self.assertAllClose(
            outputs_count / np.sum(outputs_count),
            [0.0, 0.0, 0.0, 1.0],
            rtol=0.2,
        )

    def test_only_choose_from_top_k_tokens(self):
        # Test that there are only the top-k tokens in the output.
        def token_probability_fn(inputs):
            batch_size = inputs.shape[0]
            prob = tf.constant([[0.4, 0.3, 0.2, 0.1]])
            return tf.repeat(prob, batch_size, axis=0)

        # Test that it only samples from top-k tokens.
        for k in [1, 2, 3]:
            inputs = tf.constant([[0, 0], [0, 0]])
            for _ in range(10):
                outputs = top_k_search(
                    token_probability_fn,
                    inputs,
                    max_length=5,
                    k=k,
                )
                self.assertAllEqual(outputs < k, tf.ones_like(outputs))

    def test_end_token_id(self):
        def token_probability_fn(inputs):
            batch_size = inputs.shape[0]
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, batch_size, axis=0)

        max_length = 5
        inputs = tf.constant([[0, 1], [1, 2]])
        tf.random.set_seed(42)
        outputs = top_k_search(
            token_probability_fn,
            inputs,
            max_length=max_length,
            k=4,
            seed=42,
            end_token_id=2,
            pad_token_id=0,
        )
        # Top-k sampling result with seed 42.
        expected_outputs = tf.tile([[3], [0]], [1, max_length - 2])
        expected_outputs = tf.concat([inputs, expected_outputs], axis=1)
        self.assertAllEqual(outputs, expected_outputs)

    @parameterized.named_parameters(
        ("dense_jit_compile_false", False, False),
        ("dense_jit_compile_true", True, False),
        ("ragged_jit_compile_false", False, True),
    )
    def test_model_compile(self, jit_compile, ragged):
        def token_probability_fn(inputs):
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, 2, axis=0)

        max_length = 5

        class TestModel(tf.keras.Model):
            def call(self, inputs):
                generated = top_k_search(
                    token_probability_fn,
                    inputs,
                    max_length=max_length,
                    k=2,
                    seed=42,
                    end_token_id=2,
                    pad_token_id=0,
                )
                return generated

        if ragged:
            inputs = tf.ragged.constant([[0, 1], [1, 1, 2]])
            expected_outputs = tf.constant([[0, 1, 3, 3, 3], [1, 1, 2, 0, 0]])
        else:
            inputs = tf.constant([[0, 1], [1, 2]])
            expected_outputs = [[0, 1, 3, 3, 3], [1, 2, 0, 0, 0]]

        model = TestModel()
        model.compile(jit_compile=jit_compile)

        outputs = model.predict(inputs)
        self.assertAllEqual(outputs, expected_outputs)

    @parameterized.named_parameters(
        ("dense_jit_compile_false", False, False),
        ("dense_jit_compile_true", True, False),
        ("ragged_jit_compile_false", False, True),
    )
    def test_model_compile_batched_ds(self, jit_compile, ragged):
        def token_probability_fn(inputs):
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, 2, axis=0)

        max_length = 5

        class TestModel(tf.keras.Model):
            def call(self, inputs):
                generated = top_k_search(
                    token_probability_fn,
                    inputs,
                    max_length=max_length,
                    k=2,
                    seed=42,
                    end_token_id=2,
                    pad_token_id=0,
                )
                return generated

        if ragged:
            inputs = tf.ragged.constant([[0, 1], [1, 1, 2]])
            expected_outputs = tf.constant([[0, 1, 3, 3, 3], [1, 1, 2, 0, 0]])
        else:
            inputs = tf.constant([[0, 1], [1, 2]])
            expected_outputs = [[0, 1, 3, 3, 3], [1, 2, 0, 0, 0]]

        ds = tf.data.Dataset.from_tensor_slices(inputs).batch(2)

        model = TestModel()
        model.compile(jit_compile=jit_compile)

        outputs = model.predict(ds)
        self.assertAllEqual(outputs, expected_outputs)

    def test_from_logits(self):
        def token_logits_fn(inputs):
            batch_size = inputs.shape[0]
            prob = tf.constant([[1.0, 2.0, 3.0, 4.0]])
            return tf.repeat(prob, batch_size, axis=0)

        def token_probability_fn(inputs):
            batch_size = inputs.shape[0]
            prob = keras.activations.softmax(
                tf.constant([[1.0, 2.0, 3.0, 4.0]])
            )
            return tf.repeat(prob, batch_size, axis=0)

        max_length = 5
        inputs = tf.constant([[0, 1], [1, 2]])
        tf.random.set_seed(42)
        output_logit = top_k_search(
            token_logits_fn,
            inputs,
            max_length=max_length,
            k=3,
            from_logits=True,
            seed=42,
        )
        tf.random.set_seed(42)
        output_probs = top_k_search(
            token_probability_fn,
            inputs,
            max_length=max_length,
            k=3,
            from_logits=False,
            seed=42,
        )
        self.assertAllEqual(output_logit, output_probs)


class TopPSearchTextGenerationTest(tf.test.TestCase, parameterized.TestCase):
    def setUp(self):
        super().setUp()
        self.vocab_size = 10
        self.feature_size = 16

        # Create a dummy model to predict the next token.
        model = keras.Sequential(
            [
                keras.Input(shape=[None]),
                keras.layers.Embedding(
                    input_dim=self.vocab_size,
                    output_dim=self.feature_size,
                ),
                keras.layers.Dense(self.vocab_size),
                keras.layers.Softmax(),
            ]
        )

        def token_probability_fn(inputs):
            return model(inputs)[:, -1, :]

        self.token_probability_fn = token_probability_fn

    def test_generate_with_1d_prompt(self):
        inputs = tf.constant([1])
        outputs = top_p_search(
            self.token_probability_fn, inputs, max_length=5, p=0.8
        )
        self.assertEqual(outputs.shape, [5])

    def test_generate_with_2d_prompt(self):
        inputs = tf.constant([[1], [1]])
        outputs = top_p_search(
            self.token_probability_fn, inputs, max_length=5, p=0.8
        )
        self.assertEqual(outputs.shape, [2, 5])

    def test_generate_with_list_prompt(self):
        inputs = [[1], [1]]
        outputs = top_p_search(
            self.token_probability_fn, inputs, max_length=5, p=0.8
        )
        self.assertEqual(outputs.shape, [2, 5])

    def test_generate_with_ragged_prompt(self):
        def token_probability_fn(inputs):
            # Assert that user function is passed only dense tensors.
            self.assertIsInstance(inputs, tf.Tensor)
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, 2, axis=0)

        inputs = tf.ragged.constant([[1], [2, 1, 2]])
        outputs = top_p_search(
            token_probability_fn, inputs, max_length=5, p=0.8
        )
        self.assertEqual(outputs.shape, [2, 5])

    def test_assert_probability_distribution_generation_is_correct(self):
        def token_probability_fn(inputs):
            batch_size = inputs.shape[0]
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, batch_size, axis=0)

        batch_size = 10
        inputs = 3 * tf.ones([batch_size, 1], dtype=tf.int32)
        max_length = 3

        outputs_count = np.array([0, 0, 0, 0])
        tf.random.set_seed(42)
        for i in range(64):
            outputs = top_p_search(
                token_probability_fn,
                inputs,
                max_length=max_length,
                p=0.6,
                seed=42,
            )
            flatten_predictions = tf.reshape(outputs[:, 1:], [-1])
            for pred in flatten_predictions:
                outputs_count[pred] += 1
        self.assertAllClose(
            outputs_count / np.sum(outputs_count),
            [0.0, 0.0, 0.0, 1.0],
            rtol=0.2,
        )

    def test_only_choose_from_top_p_tokens(self):
        def token_probability_fn(inputs):
            batch_size = inputs.shape[0]
            prob = tf.constant([[0.4, 0.3, 0.2, 0.1]])
            return tf.repeat(prob, batch_size, axis=0)

        # Test that it only samples from tokens that sum up to p.
        for p, n in [(0.3, 1), (0.7, 2), (0.9, 3)]:
            inputs = tf.constant([[0, 0], [0, 0]])
            for _ in range(10):
                outputs = top_p_search(
                    token_probability_fn, inputs, max_length=5, p=p
                )
                self.assertAllEqual(outputs < n, tf.ones_like(outputs))

    def test_end_token_id(self):
        def token_probability_fn(inputs):
            batch_size = inputs.shape[0]
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, batch_size, axis=0)

        max_length = 5
        inputs = tf.constant([[0, 1], [1, 2]])
        tf.random.set_seed(42)
        outputs = top_p_search(
            token_probability_fn,
            inputs,
            max_length=max_length,
            p=0.92,
            seed=1,
            end_token_id=2,
            pad_token_id=0,
        )
        # Top-p sampling result with seed 42.
        expected_outputs = tf.tile([[3], [0]], [1, max_length - 2])
        expected_outputs = tf.concat([inputs, expected_outputs], axis=1)
        self.assertAllEqual(outputs, expected_outputs)

    @parameterized.named_parameters(
        ("dense_jit_compile_false", False, False),
        ("dense_jit_compile_true", True, False),
        ("ragged_jit_compile_false", False, True),
    )
    def test_model_compile(self, jit_compile, ragged):
        def token_probability_fn(inputs):
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, 2, axis=0)

        max_length = 5

        class TestModel(tf.keras.Model):
            def call(self, inputs):
                generated = top_p_search(
                    token_probability_fn,
                    inputs,
                    max_length=max_length,
                    p=0.92,
                    seed=1,
                    end_token_id=2,
                    pad_token_id=0,
                )
                return generated

        if ragged:
            inputs = tf.ragged.constant([[0, 1], [1, 1, 2]])
            expected_outputs = tf.constant([[0, 1, 3, 3, 3], [1, 1, 2, 0, 0]])
        else:
            inputs = tf.constant([[0, 1], [1, 2]])
            expected_outputs = [[0, 1, 3, 3, 3], [1, 2, 0, 0, 0]]

        model = TestModel()
        model.compile(jit_compile=jit_compile)

        outputs = model.predict(inputs)
        self.assertAllEqual(outputs, expected_outputs)

    @parameterized.named_parameters(
        ("dense_jit_compile_false", False, False),
        ("dense_jit_compile_true", True, False),
        ("ragged_jit_compile_false", False, True),
    )
    def test_model_compile_batched_ds(self, jit_compile, ragged):
        def token_probability_fn(inputs):
            prob = tf.constant([[0.0, 0.0, 0.0, 1.0]])
            return tf.repeat(prob, 2, axis=0)

        max_length = 5

        class TestModel(tf.keras.Model):
            def call(self, inputs):
                generated = top_p_search(
                    token_probability_fn,
                    inputs,
                    max_length=max_length,
                    p=0.92,
                    seed=1,
                    end_token_id=2,
                    pad_token_id=0,
                )
                return generated

        if ragged:
            inputs = tf.ragged.constant([[0, 1], [1, 1, 2]])
            expected_outputs = tf.constant([[0, 1, 3, 3, 3], [1, 1, 2, 0, 0]])
        else:
            inputs = tf.constant([[0, 1], [1, 2]])
            expected_outputs = [[0, 1, 3, 3, 3], [1, 2, 0, 0, 0]]

        ds = tf.data.Dataset.from_tensor_slices(inputs).batch(2)

        model = TestModel()
        model.compile(jit_compile=jit_compile)

        outputs = model.predict(ds)
        self.assertAllEqual(outputs, expected_outputs)

    def test_from_logits(self):
        def token_logits_fn(inputs):
            batch_size = inputs.shape[0]
            prob = tf.constant([[1.0, 2.0, 3.0, 4.0]])
            return tf.repeat(prob, batch_size, axis=0)

        def token_probability_fn(inputs):
            batch_size = inputs.shape[0]
            prob = keras.activations.softmax(
                tf.constant([[1.0, 2.0, 3.0, 4.0]])
            )
            return tf.repeat(prob, batch_size, axis=0)

        max_length = 5
        inputs = tf.constant([[0, 1], [1, 2]])
        tf.random.set_seed(42)
        output_logit = top_p_search(
            token_logits_fn,
            inputs,
            max_length=max_length,
            p=0.92,
            from_logits=True,
            seed=42,
        )
        tf.random.set_seed(42)
        output_probs = top_p_search(
            token_probability_fn,
            inputs,
            max_length=max_length,
            p=0.92,
            from_logits=False,
            seed=42,
        )
        self.assertAllEqual(output_logit, output_probs)



# Copyright 2023 The KerasNLP Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for loading pretrained model presets."""

import pytest
import tensorflow as tf
from absl.testing import parameterized

from keras_nlp.models.opt.opt_backbone import OPTBackbone
from keras_nlp.models.opt.opt_tokenizer import OPTTokenizer


@pytest.mark.large
class OPTPresetSmokeTest(tf.test.TestCase, parameterized.TestCase):
    """
    A smoke test for GPT-2 presets we run continuously.

    This only tests the smallest weights we have available. Run with:
    `pytest keras_nlp/models/opt/opt_presets_test.py --run_large`
    """

    def test_tokenizer_output(self):
        tokenizer = OPTTokenizer.from_preset("opt_125m_en")
        outputs = tokenizer("The quick brown fox.")
        expected_outputs = [133, 2119, 6219, 23602, 4]
        self.assertAllEqual(outputs, expected_outputs)

    @parameterized.named_parameters(
        ("preset_weights", True), ("random_weights", False)
    )
    def test_backbone_output(self, load_weights):
        input_data = {
            "token_ids": tf.constant([[133, 2119, 6219, 23602, 4]]),
            "padding_mask": tf.constant([[1, 1, 1, 1, 1]]),
        }
        model = OPTBackbone.from_preset(
            "opt_125m_en", load_weights=load_weights
        )
        outputs = model(input_data)[0, 0, :5]
        if load_weights:
            # The forward pass from a preset should be stable!
            # This test should catch cases where we unintentionally change our
            # network code in a way that would invalidate our preset weights.
            # We should only update these numbers if we are updating a weights
            # file, or have found a discrepancy with the upstream source.
            expected_outputs = [-0.246, -1.004, -0.072, 0.097, 0.533]
            # Keep a high tolerance, so we are robust to different hardware.
            self.assertAllClose(outputs, expected_outputs, atol=0.01, rtol=0.01)

    @parameterized.named_parameters(
        ("opt_tokenizer", OPTTokenizer),
        ("opt_backbone", OPTBackbone),
    )
    def test_preset_docstring(self, cls):
        """Check we did our docstring formatting correctly."""
        for name in cls.presets:
            self.assertRegex(cls.from_preset.__doc__, name)

    @parameterized.named_parameters(
        ("opt_tokenizer", OPTTokenizer),
        ("opt_backbone", OPTBackbone),
    )
    def test_unknown_preset_error(self, cls):
        # Not a preset name
        with self.assertRaises(ValueError):
            cls.from_preset("opt_clowntown")


@pytest.mark.extra_large
class OPTPresetFullTest(tf.test.TestCase, parameterized.TestCase):
    """
    Test the full enumeration of our preset.

    This tests every GPT-2 preset and is only run manually.
    Run with:
    `pytest keras_nlp/models/opt/opt_presets_test.py --run_extra_large`
    """

    @parameterized.named_parameters(
        ("preset_weights", True), ("random_weights", False)
    )
    def test_load_opt(self, load_weights):
        for preset in OPTBackbone.presets:
            model = OPTBackbone.from_preset(preset, load_weights=load_weights)
            input_data = {
                "token_ids": tf.random.uniform(
                    shape=(1, 1024),
                    dtype=tf.int64,
                    maxval=model.vocabulary_size,
                ),
                "padding_mask": tf.constant([1] * 1024, shape=(1, 1024)),
            }
            model(input_data)

    def test_load_tokenizers(self):
        for preset in OPTTokenizer.presets:
            tokenizer = OPTTokenizer.from_preset(preset)
            tokenizer("The quick brown fox.")



# Copyright 2023 The KerasNLP Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""FNet encoder block implementation based on `keras.layers.Layer`."""

import tensorflow as tf
from tensorflow import keras

from keras_nlp.utils.keras_utils import clone_initializer


@keras.utils.register_keras_serializable(package="keras_nlp")
class FNetEncoder(keras.layers.Layer):
    """FNet encoder.

    This class follows the architecture of FNet encoder layer in the
    [FNet paper](https://arxiv.org/abs/2105.03824). Users can instantiate
    multiple instances of this class to stack up the encoder.

    Note on masking: In the official FNet code, padding tokens are added to the
    the input. However, the padding masks are deleted, i.e., mixing of
    all tokens is done. This is because certain frequencies will be zeroed
    out if we apply padding masks in every encoder layer. Hence, we don't
    take padding mask as input in the call() function.

    Args:
        intermediate_dim: int. The hidden size of feedforward network.
        dropout: float, defaults to 0. The dropout value, applied in the
            feedforward network.
        activation: string or `keras.activations`, defaults to "relu". The
            activation function of feedforward network.
        layer_norm_epsilon: float, defaults to 1e-5. The epsilon value in layer
            normalization components.
        kernel_initializer: "string" or `keras.initializers` initializer,
            defaults to "glorot_uniform". The kernel initializer for the dense
            layers.
        bias_initializer: "string" or `keras.initializers` initializer,
            defaults to "zeros". The bias initializer for the dense layers.
        name: string, defaults to None. The name of the layer.
        **kwargs: other keyword arguments.

    Examples:

    ```python
    # Create a single FNet encoder layer.
    encoder = keras_nlp.layers.FNetEncoder(
        intermediate_dim=64)

    # Create a simple model containing the encoder.
    input = keras.Input(shape=[10, 64])
    output = encoder(input)
    model = keras.Model(inputs=input, outputs=output)

    # Call encoder on the inputs.
    input_data = tf.random.uniform(shape=[1, 10, 64])
    output = model(input_data)
    ```

    References:
     - [Lee-Thorp et al., 2021](https://arxiv.org/abs/2105.03824)
    """

    def __init__(
        self,
        intermediate_dim,
        dropout=0,
        activation="relu",
        layer_norm_epsilon=1e-5,
        kernel_initializer="glorot_uniform",
        bias_initializer="zeros",
        name=None,
        **kwargs
    ):
        super().__init__(name=name, **kwargs)
        self.intermediate_dim = intermediate_dim
        self.dropout = dropout
        self.activation = keras.activations.get(activation)
        self.layer_norm_epsilon = layer_norm_epsilon
        self.kernel_initializer = keras.initializers.get(kernel_initializer)
        self.bias_initializer = keras.initializers.get(bias_initializer)

    def build(self, input_shape):
        # Create layers based on input shape.
        feature_size = input_shape[-1]

        # Layer Norm layers.
        self._mixing_layer_norm = keras.layers.LayerNormalization(
            epsilon=self.layer_norm_epsilon
        )
        self._output_layer_norm = keras.layers.LayerNormalization(
            epsilon=self.layer_norm_epsilon
        )

        # Feedforward layers.
        self._intermediate_dense = keras.layers.Dense(
            self.intermediate_dim,
            activation=self.activation,
            kernel_initializer=clone_initializer(self.kernel_initializer),
            bias_initializer=clone_initializer(self.bias_initializer),
        )
        self._output_dense = keras.layers.Dense(
            feature_size,
            kernel_initializer=clone_initializer(self.kernel_initializer),
            bias_initializer=clone_initializer(self.bias_initializer),
        )
        self._output_dropout = keras.layers.Dropout(rate=self.dropout)

    def call(self, inputs):
        """Forward pass of the FNetEncoder.

        Args:
            inputs: a Tensor. The input data to TransformerEncoder, should be
                of shape [batch_size, sequence_length, feature_dim].

        Returns:
            A Tensor of the same shape as the `inputs`.
        """

        def fourier_transform(input):
            # Apply FFT on the input and take the real part.
            # Before we apply fourier transform, let's convert the dtype of the
            # input tensor to complex64.
            input = tf.cast(input, tf.complex64)
            mixing_output = tf.math.real(tf.signal.fft2d(input))
            return mixing_output

        def add_and_norm(input1, input2, norm_layer):
            return norm_layer(input1 + input2)

        def feed_forward(input):
            x = self._intermediate_dense(input)
            x = self._output_dense(x)
            return self._output_dropout(x)

        mixing_output = fourier_transform(inputs)

        mixing_output = add_and_norm(
            inputs, mixing_output, self._mixing_layer_norm
        )

        feed_forward_output = feed_forward(mixing_output)

        x = add_and_norm(
            mixing_output, feed_forward_output, self._output_layer_norm
        )
        return x

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "intermediate_dim": self.intermediate_dim,
                "dropout": self.dropout,
                "activation": keras.activations.serialize(self.activation),
                "layer_norm_epsilon": self.layer_norm_epsilon,
                "kernel_initializer": keras.initializers.serialize(
                    self.kernel_initializer
                ),
                "bias_initializer": keras.initializers.serialize(
                    self.bias_initializer
                ),
            }
        )
        return config



# Copyright 2023 The KerasNLP Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Transformer decoder block implementation based on `keras.layers.Layer`."""

import tensorflow as tf
from tensorflow import keras

from keras_nlp.utils.keras_utils import clone_initializer

from keras_nlp.layers.transformer_layer_utils import (  # isort:skip
    compute_causal_mask,
    merge_padding_and_attention_mask,
)


@keras.utils.register_keras_serializable(package="keras_nlp")
class TransformerDecoder(keras.layers.Layer):
    """Transformer decoder.

    This class follows the architecture of the transformer decoder layer in the
    paper [Attention is All You Need](https://arxiv.org/abs/1706.03762). Users
    can instantiate multiple instances of this class to stack up a decoder.

    This layer will always apply a causal mask to the decoder attention layer.
    This layer will correctly compute an attention mask from an implicit
    Keras padding mask (for example, by passing `mask_zero=True` to a
    `keras.layers.Embedding` layer). See the Masking and Padding
    [guide](https://keras.io/guides/understanding_masking_and_padding/)
    for more details.

    This layer can be called with either one or two inputs. The number of inputs
    must be consistent across all calls. The options are as follows:
        `layer(decoder_sequence)`: no cross-attention will be built into the
            decoder block. This is useful when building a "decoder-only"
            transformer such as GPT-2.
        `layer(decoder_sequence, encoder_sequence)`: cross-attention will be
            built into the decoder block. This is useful when building an
            "encoder-decoder" transformer, such as the original transformer
            model described in Attention is All You Need.

    Args:
        intermediate_dim: int, the hidden size of feedforward network.
        num_heads: int, the number of heads in MultiHeadAttention.
        dropout: float, defaults to 0. the dropout value, shared by
            MultiHeadAttention and feedforward network.
        activation: string or `keras.activations`, defaults to "relu". the
            activation function of feedforward network.
        layer_norm_epsilon: float, defaults to 1e-5. The eps value in layer
            normalization components.
        kernel_initializer: string or `keras.initializers` initializer,
            defaults to "glorot_uniform". The kernel initializer for
            the dense and multiheaded attention layers.
        bias_initializer: string or `keras.initializers` initializer,
            defaults to "zeros". The bias initializer for
            the dense and multiheaded attention layers.
        normalize_first: bool. Defaults to False. If True, the inputs to the
            attention layer(s) and the intermediate dense layer are normalized
            (similar to GPT-2). If set to False, outputs of attention layer and
            intermediate dense layer are normalized (similar to BERT).
        name: string, defaults to None. The name of the layer.
        **kwargs: other keyword arguments.

    Examples:
    ```python
    # Create a single transformer decoder layer.
    decoder = keras_nlp.layers.TransformerDecoder(
        intermediate_dim=64, num_heads=8)

    # Create a simple model containing the decoder.
    decoder_input = keras.Input(shape=[10, 64])
    encoder_input = keras.Input(shape=[10, 64])
    output = decoder(decoder_input, encoder_input)
    model = keras.Model(inputs=[decoder_input, encoder_input],
        outputs=output)

    # Call decoder on the inputs.
    decoder_input_data = tf.random.uniform(shape=[2, 10, 64])
    encoder_input_data = tf.random.uniform(shape=[2, 10, 64])
    decoder_output = model([decoder_input_data, encoder_input_data])

    ```

    References:
     - [Vaswani et al., 2017](https://arxiv.org/abs/1706.03762)

    """

    def __init__(
        self,
        intermediate_dim,
        num_heads,
        dropout=0,
        activation="relu",
        layer_norm_epsilon=1e-05,
        kernel_initializer="glorot_uniform",
        bias_initializer="zeros",
        normalize_first=False,
        name=None,
        **kwargs,
    ):
        # Work around for model saving, we need to ensure our model is built
        # immediately after restoring from config.
        self._input_shape = kwargs.pop("build_input_shape", None)
        self._has_cross_attention = kwargs.pop("has_cross_attention", False)

        super().__init__(name=name, **kwargs)
        self.intermediate_dim = intermediate_dim
        self.num_heads = num_heads
        self.dropout = dropout
        self.activation = keras.activations.get(activation)
        self.layer_norm_epsilon = layer_norm_epsilon
        self.kernel_initializer = keras.initializers.get(kernel_initializer)
        self.bias_initializer = keras.initializers.get(bias_initializer)
        self.normalize_first = normalize_first
        self._built = False
        self.supports_masking = True

        if self._input_shape is not None:
            self._build(self._input_shape, self._has_cross_attention)

    def _build(self, input_shape, has_cross_attention):
        # Create layers based on input shape.
        self._built = True
        self._input_shape = input_shape
        self._has_cross_attention = has_cross_attention
        # Infer the dimension of our hidden feature size from the build shape.
        hidden_dim = input_shape[-1]
        # Attention head size is `hidden_dim` over the number of heads.
        head_dim = int(hidden_dim // self.num_heads)

        # Self attention layers.
        self._self_attention_layer = keras.layers.MultiHeadAttention(
            num_heads=self.num_heads,
            key_dim=head_dim,
            dropout=self.dropout,
            kernel_initializer=clone_initializer(self.kernel_initializer),
            bias_initializer=clone_initializer(self.bias_initializer),
        )
        self._self_attention_layer._build_from_signature(
            query=input_shape,
            value=input_shape,
        )
        self._self_attention_layernorm = keras.layers.LayerNormalization(
            epsilon=self.layer_norm_epsilon,
        )
        self._self_attention_dropout = keras.layers.Dropout(
            rate=self.dropout,
        )

        # Cross attention layers are optional.
        self._cross_attention_layer = None
        if has_cross_attention:
            self._cross_attention_layer = keras.layers.MultiHeadAttention(
                num_heads=self.num_heads,
                key_dim=head_dim,
                value_dim=head_dim,
                dropout=self.dropout,
                kernel_initializer=clone_initializer(self.kernel_initializer),
                bias_initializer=clone_initializer(self.bias_initializer),
            )
            self._cross_attention_layer._build_from_signature(
                query=input_shape,
                value=input_shape,
            )
            self._cross_attention_layernorm = keras.layers.LayerNormalization(
                epsilon=self.layer_norm_epsilon,
            )
            self._cross_attention_dropout = keras.layers.Dropout(
                rate=self.dropout,
            )

        # Feedforward layers.
        self._feedforward_intermediate_dense = keras.layers.Dense(
            self.intermediate_dim,
            activation=self.activation,
            kernel_initializer=clone_initializer(self.kernel_initializer),
            bias_initializer=clone_initializer(self.bias_initializer),
        )
        self._feedforward_output_dense = keras.layers.Dense(
            hidden_dim,
            kernel_initializer=clone_initializer(self.kernel_initializer),
            bias_initializer=clone_initializer(self.bias_initializer),
        )
        self._feedforward_layernorm = keras.layers.LayerNormalization(
            epsilon=self.layer_norm_epsilon,
        )
        self._feedforward_dropout = keras.layers.Dropout(
            rate=self.dropout,
        )

    def call(
        self,
        decoder_sequence,
        encoder_sequence=None,
        decoder_padding_mask=None,
        decoder_attention_mask=None,
        encoder_padding_mask=None,
        encoder_attention_mask=None,
    ):
        """Forward pass of the TransformerDecoder.

        Args:
            decoder_sequence: a Tensor. The decoder input sequence.
            encoder_sequence: a Tensor. The encoder input sequence. For decoder
                only models (like GPT2), this should be left None. Once the
                model is called once without an encoder_sequence, you cannot
                call it again with encoder_sequence.
            decoder_padding_mask: a boolean Tensor, the padding mask of decoder
                sequence, must of shape [batch_size, decoder_sequence_length].
            decoder_attention_mask: a boolean Tensor. Customized decoder
                sequence mask, must of shape
                [batch_size, decoder_sequence_length, decoder_sequence_length].
            encoder_padding_mask: a boolean Tensor, the padding mask of encoder
                sequence, must of shape [batch_size, encoder_sequence_length].
            encoder_attention_mask: a boolean Tensor. Customized encoder
                sequence mask, must of shape
                [batch_size, encoder_sequence_length, encoder_sequence_length].
        Returns:
            A Tensor of the same shape as the `decoder_sequence`.
        """

        has_encoder_sequence = encoder_sequence is not None
        if not self._built:
            self._build(decoder_sequence.shape, has_encoder_sequence)

        is_cross_attention = self._cross_attention_layer is not None
        if not is_cross_attention and has_encoder_sequence:
            raise ValueError(
                "The number of call arguments to "
                "`keras_nlp.layers.TransformerDecoder` should not change. "
                "Use `layer(decoder_sequence, encoder_sequence)` to "
                "build a layer with cross attention, or "
                "`layer(decoder_sequence)` to build a layer without. "
                "This layer has been built without cross attention, but "
                "you are trying to call it with encoder_sequence."
            )
        elif is_cross_attention and not has_encoder_sequence:
            raise ValueError(
                "The number of call arguments to "
                "`keras_nlp.layers.TransformerDecoder` should not change. "
                "Use `layer(decoder_sequence, encoder_sequence)` to "
                "build a layer with cross attention, or "
                "`layer(decoder_sequence)` to build a layer without. "
                "This layer has been built with cross attention, but "
                "you did not provide encoder_sequence."
            )

        x = decoder_sequence  # Intermediate result.

        # Compute self attention mask.
        self_attention_mask = compute_causal_mask(decoder_sequence)
        decoder_mask = merge_padding_and_attention_mask(
            decoder_sequence, decoder_padding_mask, decoder_attention_mask
        )
        if decoder_mask is not None:
            self_attention_mask = tf.minimum(decoder_mask, self_attention_mask)

        # Self attention block.
        residual = x
        if self.normalize_first:
            x = self._self_attention_layernorm(x)
        x = self._self_attention_layer(
            query=x,
            value=x,
            attention_mask=self_attention_mask,
        )
        x = self._self_attention_dropout(x)
        x = x + residual
        if not self.normalize_first:
            x = self._self_attention_layernorm(x)

        # Cross attention is optional.
        if self._cross_attention_layer is not None:
            # Compute cross attention mask.
            cross_attention_mask = merge_padding_and_attention_mask(
                encoder_sequence, encoder_padding_mask, encoder_attention_mask
            )

            # Cross attention block.
            residual = x
            if self.normalize_first:
                x = self._cross_attention_layernorm(x)
            x = self._cross_attention_layer(
                query=x,
                value=encoder_sequence,
                attention_mask=cross_attention_mask,
            )
            x = self._cross_attention_dropout(x)
            x = x + residual
            if not self.normalize_first:
                x = self._cross_attention_layernorm(x)

        # Feedforward block.
        residual = x
        if self.normalize_first:
            x = self._feedforward_layernorm(x)
        x = self._feedforward_intermediate_dense(x)
        x = self._feedforward_output_dense(x)
        x = self._feedforward_dropout(x)
        x = x + residual
        if not self.normalize_first:
            x = self._feedforward_layernorm(x)

        return x

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "intermediate_dim": self.intermediate_dim,
                "num_heads": self.num_heads,
                "dropout": self.dropout,
                "activation": keras.activations.serialize(self.activation),
                "layer_norm_epsilon": self.layer_norm_epsilon,
                "kernel_initializer": keras.initializers.serialize(
                    self.kernel_initializer
                ),
                "bias_initializer": keras.initializers.serialize(
                    self.bias_initializer
                ),
                "normalize_first": self.normalize_first,
                "build_input_shape": self._input_shape,
                "has_cross_attention": self._has_cross_attention,
            }
        )
        return config



# Copyright 2023 The KerasNLP Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for Sampler classes."""

import tensorflow as tf

import keras_nlp
from keras_nlp.samplers.greedy_sampler import GreedySampler


class SamplerTest(tf.test.TestCase):
    def test_serialization(self):
        sampler = keras_nlp.samplers.GreedySampler()
        config = keras_nlp.samplers.serialize(sampler)
        expected_config = {
            "class_name": "keras_nlp>GreedySampler",
            "config": {
                "jit_compile": True,
            },
        }
        self.assertDictEqual(expected_config, config)

    def test_deserialization(self):
        # Test get from string.
        identifier = "greedy"
        sampler = keras_nlp.samplers.get(identifier)
        self.assertIsInstance(sampler, GreedySampler)

        # Test dict identifier.
        original_sampler = keras_nlp.samplers.GreedySampler(jit_compile=False)
        config = keras_nlp.samplers.serialize(original_sampler)
        restored_sampler = keras_nlp.samplers.get(config)
        self.assertDictEqual(
            keras_nlp.samplers.serialize(restored_sampler),
            keras_nlp.samplers.serialize(original_sampler),
        )

        # Test identifier is already a sampler instance.
        original_sampler = keras_nlp.samplers.GreedySampler(jit_compile=False)
        restored_sampler = keras_nlp.samplers.get(original_sampler)
        self.assertEqual(original_sampler, restored_sampler)



# Copyright 2023 The KerasNLP Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Beam Sampler."""

import tensorflow as tf
from tensorflow import keras

from keras_nlp.samplers.sampler import Sampler
from keras_nlp.samplers.sampler import base_sampler_args_docstring
from keras_nlp.samplers.sampler import call_args_docstring
from keras_nlp.utils.python_utils import format_docstring


@format_docstring(
    base_sampler_args=base_sampler_args_docstring, call_args=call_args_docstring
)
@keras.utils.register_keras_serializable(package="keras_nlp")
class BeamSampler(Sampler):
    """Beam Sampler class.

    This sampler implements beam search algorithm. At each time-step, beam
    search keeps the beams (sequences) of the top `num_beams` highest
    accumulated probabilities, and uses each one of the beams to predict
    candidate next tokens.

    Args:
        num_beams: int. The number of beams that should be kept at each
            time-step. `num_beams` should be strictly positive.
        {{base_sampler_args}}

    Call Args:
        {{call_args}}

    Examples:
    ```python
    VOCAB_SIZE = 10

    # Create a dummy model to predict the next token.
    model = keras.Sequential(
        [
            keras.Input(shape=[None]),
            keras.layers.Embedding(
                input_dim=VOCAB_SIZE,
                output_dim=16,
            ),
            keras.layers.Dense(VOCAB_SIZE, activation="softmax"),
        ]
    )

    # Define a function that outputs the next token's probability for each token
    # in the input sequence.
    def token_probability_fn(inputs, mask):
        return model(inputs)

    prompt = tf.fill((8, 1), 1)

    sampler = keras_nlp.samplers.BeamSampler(num_beams=3)
    # Print the generated sequence (token ids).
    print(sampler(prompt, token_probability_fn, max_length=10))
    ```
    """

    def __init__(
        self,
        num_beams=5,
        jit_compile=True,
        run_eagerly=False,
    ):
        self.num_beams = num_beams
        super().__init__(jit_compile=jit_compile, run_eagerly=run_eagerly)

    def get_next_token(self, next_token_probs):
        # Beam search overrides the whole `sample` method.
        pass

    def sample(
        self, prompt, token_probability_fn, mask, num_steps, from_logits=True
    ):
        """Sampling logic implementation.

        Because beam search uses a different loop body, we have to override the
        whole `sample` method instead of just the `get_next_token` method.
        """
        batch_size, max_length = tf.shape(prompt)[0], tf.shape(prompt)[1]
        max_length = tf.cast(max_length, num_steps.dtype)
        length = max_length - num_steps
        dummy_preds = token_probability_fn(prompt, mask=mask)
        vocab_size = tf.shape(dummy_preds)[-1]
        pred_dtype = dummy_preds.dtype

        num_beams = self.num_beams

        # Initialize beam with shape `(batch_size, num_beams, length)`.
        beams = tf.repeat(tf.expand_dims(prompt, axis=1), num_beams, axis=1)
        # Initialize `beams_prob` with shape `(batch_size, num_beams)`.
        beams_prob = tf.zeros([batch_size, 1], dtype=pred_dtype)
        beams_prob = tf.concat(
            [beams_prob, tf.fill((batch_size, num_beams - 1), pred_dtype.min)],
            axis=-1,
        )

        def one_step(beams, beams_prob, length, mask):
            flattened_beams = tf.reshape(
                beams, shape=[batch_size * num_beams, -1]
            )
            repeated_mask = tf.tile(mask, [num_beams, 1])
            probs = token_probability_fn(flattened_beams, repeated_mask)
            preds = tf.gather(
                probs,
                tf.repeat(length - 1, batch_size * num_beams),
                axis=1,
                batch_dims=1,
            )
            if from_logits:
                preds = keras.activations.softmax(preds, axis=-1)
            # Reshape `preds` to shape `(batch_size, num_beams * vocab_size)`.

            preds = tf.reshape(preds, shape=[batch_size, -1])

            cum_probs = tf.math.log(preds) + tf.repeat(
                beams_prob, repeats=vocab_size, axis=1
            )

            candidate_prob, candidate_indexes = tf.math.top_k(
                cum_probs, k=num_beams, sorted=False
            )

            candidate_beam_indexes = candidate_indexes // vocab_size
            next_token = candidate_indexes % vocab_size

            beams = tf.gather(
                beams, candidate_beam_indexes, axis=1, batch_dims=1
            )

            # Build a new column of updates to scatter into the beam tensor.
            next_token = tf.where(
                condition=mask[..., length, tf.newaxis],
                x=beams[..., length],
                y=next_token,
            )
            next_token = tf.reshape(next_token, shape=[-1])

            mask = tf.tensor_scatter_nd_update(
                tensor=mask,
                indices=tf.stack(
                    (
                        tf.cast(tf.range(batch_size), dtype=length.dtype),
                        tf.repeat(length, batch_size),
                    ),
                    axis=1,
                ),
                updates=tf.repeat(True, batch_size),
            )

            # Generate `(batch_index, beam_index)` tuples for each beam.
            beam_indices = tf.where(tf.ones((batch_size, num_beams), tf.bool))
            beam_indices = tf.cast(beam_indices, dtype=length.dtype)
            # Build a tensor of repeated `length` values.
            length_indices = tf.fill((batch_size * num_beams, 1), length)
            # Concatenate to a triplet of `(batch_index, beam_index, length)`.
            indices = tf.concat([beam_indices, length_indices], axis=-1)

            # Update `beams[:, :, length]` with `next_token`.
            beams = tf.tensor_scatter_nd_update(
                tensor=beams,
                indices=indices,
                updates=next_token,
            )

            beams_prob = candidate_prob

            length = tf.add(length, 1)
            return beams, beams_prob, length, mask

        # Run a while loop till text of length `max_length` has been generated.
        beams, beams_prob, length, mask = tf.while_loop(
            cond=lambda beams, beams_prob, length, mask: tf.less(
                length, max_length
            ),
            body=one_step,
            loop_vars=[beams, beams_prob, length, mask],
            # There is a strange issue that when `batch_size=1`, the first loop
            # iteration changes `beams_prob`'s shape from [1, None] to
            # [None, None], which does not happen for `batch_size>1`.
            # As a workaround, we set shape invariants.
            shape_invariants=[
                beams.get_shape(),
                tf.TensorShape([None, None]),
                length.get_shape(),
                mask.get_shape(),
            ],
        )

        # Get the beam with the maximum probability.
        max_indexes = tf.math.argmax(beams_prob, axis=-1)
        max_beams = tf.gather(
            beams, max_indexes[:, tf.newaxis], axis=1, batch_dims=1
        )

        return tf.squeeze(max_beams, axis=1)

    def get_config(self):
        config = super().get_config()

        config.update({"num_beams": self.num_beams})
        return config



# Copyright 2023 The KerasNLP Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import shutil

import numpy as np
import tensorflow as tf
import transformers
from absl import app
from absl import flags

import keras_nlp
from tools.checkpoint_conversion.checkpoint_conversion_utils import (
    get_md5_checksum,
)

PRESET_MAP = {
    "albert_base_en_uncased": "albert-base-v2",
    "albert_large_en_uncased": "albert-large-v2",
    "albert_extra_large_en_uncased": "albert-xlarge-v2",
    "albert_extra_extra_large_en_uncased": "albert-xxlarge-v2",
}


FLAGS = flags.FLAGS
flags.DEFINE_string(
    "preset", None, f'Must be one of {",".join(PRESET_MAP.keys())}'
)


def convert_checkpoints(hf_model):
    print("\n-> Convert original weights to KerasNLP format.")

    print("\n-> Load KerasNLP model.")
    keras_nlp_model = keras_nlp.models.AlbertBackbone.from_preset(
        FLAGS.preset, load_weights=False
    )

    hf_wts = hf_model.state_dict()
    print("Original weights:")
    print(list(hf_wts.keys()))

    num_heads = keras_nlp_model.num_heads
    hidden_dim = keras_nlp_model.hidden_dim

    keras_nlp_model.get_layer("token_embedding").embeddings.assign(
        hf_wts["embeddings.word_embeddings.weight"]
    )
    keras_nlp_model.get_layer("position_embedding").position_embeddings.assign(
        hf_wts["embeddings.position_embeddings.weight"]
    )
    keras_nlp_model.get_layer("segment_embedding").embeddings.assign(
        hf_wts["embeddings.token_type_embeddings.weight"]
    )

    keras_nlp_model.get_layer("embeddings_layer_norm").gamma.assign(
        hf_wts["embeddings.LayerNorm.weight"]
    )
    keras_nlp_model.get_layer("embeddings_layer_norm").beta.assign(
        hf_wts["embeddings.LayerNorm.bias"]
    )

    keras_nlp_model.get_layer("embedding_projection").kernel.assign(
        hf_wts["encoder.embedding_hidden_mapping_in.weight"].T
    )
    keras_nlp_model.get_layer("embedding_projection").bias.assign(
        hf_wts["encoder.embedding_hidden_mapping_in.bias"]
    )

    for i in range(keras_nlp_model.num_groups):
        for j in range(keras_nlp_model.num_inner_repetitions):
            keras_nlp_model.get_layer(
                f"group_{i}_inner_layer_{j}"
            )._self_attention_layer._query_dense.kernel.assign(
                hf_wts[
                    f"encoder.albert_layer_groups.{i}.albert_layers.{j}.attention.query.weight"
                ]
                .transpose(1, 0)
                .reshape((hidden_dim, num_heads, -1))
                .numpy()
            )
            keras_nlp_model.get_layer(
                f"group_{i}_inner_layer_{j}"
            )._self_attention_layer._query_dense.bias.assign(
                hf_wts[
                    f"encoder.albert_layer_groups.{i}.albert_layers.{j}.attention.query.bias"
                ]
                .reshape((num_heads, -1))
                .numpy()
            )

            keras_nlp_model.get_layer(
                f"group_{i}_inner_layer_{j}"
            )._self_attention_layer._key_dense.kernel.assign(
                hf_wts[
                    f"encoder.albert_layer_groups.{i}.albert_layers.{j}.attention.key.weight"
                ]
                .transpose(1, 0)
                .reshape((hidden_dim, num_heads, -1))
                .numpy()
            )
            keras_nlp_model.get_layer(
                f"group_{i}_inner_layer_{j}"
            )._self_attention_layer._key_dense.bias.assign(
                hf_wts[
                    f"encoder.albert_layer_groups.{i}.albert_layers.{j}.attention.key.bias"
                ]
                .reshape((num_heads, -1))
                .numpy()
            )

            keras_nlp_model.get_layer(
                f"group_{i}_inner_layer_{j}"
            )._self_attention_layer._value_dense.kernel.assign(
                hf_wts[
                    f"encoder.albert_layer_groups.{i}.albert_layers.{j}.attention.value.weight"
                ]
                .transpose(1, 0)
                .reshape((hidden_dim, num_heads, -1))
                .numpy()
            )
            keras_nlp_model.get_layer(
                f"group_{i}_inner_layer_{j}"
            )._self_attention_layer._value_dense.bias.assign(
                hf_wts[
                    f"encoder.albert_layer_groups.{i}.albert_layers.{j}.attention.value.bias"
                ]
                .reshape((num_heads, -1))
                .numpy()
            )

            keras_nlp_model.get_layer(
                f"group_{i}_inner_layer_{j}"
            )._self_attention_layer._output_dense.kernel.assign(
                hf_wts[
                    f"encoder.albert_layer_groups.{i}.albert_layers.{j}.attention.dense.weight"
                ]
                .transpose(1, 0)
                .reshape((num_heads, -1, hidden_dim))
                .numpy()
            )
            keras_nlp_model.get_layer(
                f"group_{i}_inner_layer_{j}"
            )._self_attention_layer._output_dense.bias.assign(
                hf_wts[
                    f"encoder.albert_layer_groups.{i}.albert_layers.{j}.attention.dense.bias"
                ].numpy()
            )

            keras_nlp_model.get_layer(
                f"group_{i}_inner_layer_{j}"
            )._self_attention_layernorm.gamma.assign(
                hf_wts[
                    f"encoder.albert_layer_groups.{i}.albert_layers.{j}.attention.LayerNorm.weight"
                ].numpy()
            )
            keras_nlp_model.get_layer(
                f"group_{i}_inner_layer_{j}"
            )._self_attention_layernorm.beta.assign(
                hf_wts[
                    f"encoder.albert_layer_groups.{i}.albert_layers.{j}.attention.LayerNorm.bias"
                ].numpy()
            )

            keras_nlp_model.get_layer(
                f"group_{i}_inner_layer_{j}"
            )._feedforward_intermediate_dense.kernel.assign(
                hf_wts[
                    f"encoder.albert_layer_groups.{i}.albert_layers.{j}.ffn.weight"
                ]
                .transpose(1, 0)
                .numpy()
            )
            keras_nlp_model.get_layer(
                f"group_{i}_inner_layer_{j}"
            )._feedforward_intermediate_dense.bias.assign(
                hf_wts[
                    f"encoder.albert_layer_groups.{i}.albert_layers.{j}.ffn.bias"
                ].numpy()
            )

            keras_nlp_model.get_layer(
                f"group_{i}_inner_layer_{j}"
            )._feedforward_output_dense.kernel.assign(
                hf_wts[
                    f"encoder.albert_layer_groups.{i}.albert_layers.{j}.ffn_output.weight"
                ]
                .transpose(1, 0)
                .numpy()
            )
            keras_nlp_model.get_layer(
                f"group_{i}_inner_layer_{j}"
            )._feedforward_output_dense.bias.assign(
                hf_wts[
                    f"encoder.albert_layer_groups.{i}.albert_layers.{j}.ffn_output.bias"
                ].numpy()
            )

            keras_nlp_model.get_layer(
                f"group_{i}_inner_layer_{j}"
            )._feedforward_layernorm.gamma.assign(
                hf_wts[
                    f"encoder.albert_layer_groups.{i}.albert_layers.{j}.full_layer_layer_norm.weight"
                ].numpy()
            )
            keras_nlp_model.get_layer(
                f"group_{i}_inner_layer_{j}"
            )._feedforward_layernorm.beta.assign(
                hf_wts[
                    f"encoder.albert_layer_groups.{i}.albert_layers.{j}.full_layer_layer_norm.bias"
                ].numpy()
            )

    keras_nlp_model.get_layer("pooled_dense").kernel.assign(
        hf_wts["pooler.weight"].transpose(1, 0).numpy()
    )
    keras_nlp_model.get_layer("pooled_dense").bias.assign(
        hf_wts["pooler.bias"].numpy()
    )

    # Save the model.
    print("\n-> Save KerasNLP model weights.")
    keras_nlp_model.save_weights(os.path.join(FLAGS.preset, "model.h5"))

    return keras_nlp_model


def extract_vocab(hf_tokenizer):
    spm_path = os.path.join(FLAGS.preset, "spiece.model")
    print(f"\n-> Save KerasNLP SPM vocabulary file to `{spm_path}`.")

    shutil.copyfile(
        transformers.utils.hub.get_file_from_repo(
            hf_tokenizer.name_or_path, "spiece.model"
        ),
        spm_path,
    )

    keras_nlp_tokenizer = keras_nlp.models.AlbertTokenizer(
        proto=spm_path,
    )
    keras_nlp_preprocessor = keras_nlp.models.AlbertPreprocessor(
        keras_nlp_tokenizer
    )

    print("-> Print MD5 checksum of the vocab files.")
    print(f"`{spm_path}` md5sum: ", get_md5_checksum(spm_path))

    return keras_nlp_preprocessor


def check_output(
    keras_nlp_preprocessor,
    keras_nlp_model,
    hf_tokenizer,
    hf_model,
):
    print("\n-> Check the outputs.")
    sample_text = ["cricket is awesome, easily the best sport in the world!"]

    # KerasNLP
    keras_nlp_inputs = keras_nlp_preprocessor(tf.constant(sample_text))
    keras_nlp_output = keras_nlp_model.predict(keras_nlp_inputs)[
        "sequence_output"
    ]

    # HF
    hf_inputs = hf_tokenizer(
        sample_text, padding="max_length", return_tensors="pt"
    )
    hf_output = hf_model(**hf_inputs).last_hidden_state

    print("KerasNLP output:", keras_nlp_output[0, 0, :10])
    print("HF output:", hf_output[0, 0, :10])
    print("Difference:", np.mean(keras_nlp_output - hf_output.detach().numpy()))

    # Show the MD5 checksum of the model weights.
    print(
        "Model md5sum: ",
        get_md5_checksum(os.path.join(FLAGS.preset, "model.h5")),
    )


def main(_):
    os.makedirs(FLAGS.preset)

    hf_model_name = PRESET_MAP[FLAGS.preset]

    print("\n-> Load HF model and HF tokenizer.")
    hf_model = transformers.AutoModel.from_pretrained(hf_model_name)
    hf_model.eval()
    hf_tokenizer = transformers.AutoTokenizer.from_pretrained(hf_model_name)

    keras_nlp_model = convert_checkpoints(hf_model)
    print("\n -> Load KerasNLP preprocessor.")
    keras_nlp_preprocessor = extract_vocab(hf_tokenizer)

    check_output(
        keras_nlp_preprocessor,
        keras_nlp_model,
        hf_tokenizer,
        hf_model,
    )


if __name__ == "__main__":
    flags.mark_flag_as_required("preset")
    app.run(main)
